"""中间件"""

from fastapi import FastAPI, Request
from fastapi.middleware.cors import CORSMiddleware
from ...settings.config import CORS_ORIGINS, CORS_ALLOW_METHODS, CORS_ALLOW_CREDENTIALS, CORS_ALLOW_HEADERS
import time


def init_middlewares(app: FastAPI):
    """初始化中间件"""
    app.state.log.info('init_middlewares ...')

    # cors
    app.add_middleware(
        CORSMiddleware,
        allow_origins=CORS_ORIGINS,
        allow_credentials=CORS_ALLOW_CREDENTIALS,
        allow_methods=CORS_ALLOW_METHODS,
        allow_headers=CORS_ALLOW_HEADERS,
    )

    # 中间件
    #@app.middleware("http")
    async def test(request: Request, call_next):
        #app.state.log.info('中间件 ...')
        start_time = time.time()
        response = await call_next(request)
        process_time = time.time() - start_time
        response.headers["X-Process-Time"] = str(process_time)
        return response



# -*- coding: utf-8 -*-
import re
from ..reg import RegularExtrator as pre

class RegularExtrator(pre):

    def __init__(self, text='',data_list = []):
        pre.__init__(self, text, data_list)  
        self.text = text
        self.data_list = data_list
        
    def clear_list(self,pattern = r'[\s]*'):
        # 去除空格换行符等字符的txt文本列表
        data_list = self.flatten(self.data_list)
        data_list_out = []
        for item in data_list: 
            txt_drop = re.sub(pattern, '', item)       # 清除换行,制表,空格
            data_list_out.append(txt_drop)
        
        data_list_out = list(filter(None, data_list_out))
        self.data_list = data_list_out
        return self.data_list 
    
    def clear_txt(self,pattern = r'[\s]*'):
        # 去除空格换行符等字符的txt文本列表  
        txt_drop = re.sub(pattern, '', self.text)       # 清除换行,制表,空格 
        self.text = txt_drop
        return txt_drop 



# -*- coding: utf-8 -*-
'''
Created on 2020-9-29
csv 标注数据转换为 theta ner训练数据
@author: zhys513(254851907@qq.com)
'''
import json, os
from pathlib import Path
from tqdm import tqdm
import pandas as pd
from collections import Counter


def load_txts(filename):
    with open(filename, encoding='utf-8') as f:
        return f.readlines()


def load_txt(filename):
    with open(filename, encoding='utf-8') as f:
        return f.read()


data_dir = 'datasets/ccf2020_business_ner'
labels = []
train_data, test_data, txt_list = [], [], []


# 转换 csv 转换预测集时要考虑空的问题
def conver_labels(train_dir):
    # for file in tqdm(list(Path(data_dir, 'train').glob('*.ann'))[:]):
    label_path = Path(data_dir, train_dir, 'label')
    label_list = os.listdir(label_path)
    for file in label_list:
        file_name = str(file).split('.')[0]
        originalText = load_txt(Path(data_dir, train_dir, 'data', file_name + '.txt'))
        # originalText = originalText.replace(' ',',')
        if originalText[:1] == ' ': originalText = '@' + originalText[1:]
        txt_list.append(originalText)
        categorys = pd.read_csv(Path(label_path, file), encoding='UTF-8')  # sep='\t')
        output_data = {'originalText': originalText, 'entities': []}
        for _, row in categorys.iterrows():  # 1001,position,55,60,法国著名歌手
            label_type = row[1]
            start_pos = int(row[2])
            end_pos = int(row[3] + 1)
            mention = str(row[4])
            #if label_type in ['vx', 'QQ']:
            #    start_pos = start_pos - 3

            if mention != originalText[start_pos:end_pos]:
                print(label_type, mention, originalText[int(start_pos):int(end_pos)], file)
                # print(entity,originalText[start_pos:end_pos])
            output_data['entities'].append({
                'label_type': label_type,
                'overlap': 0,
                'start_pos': start_pos,
                'end_pos': end_pos,
                'mention': originalText[start_pos:end_pos]
            })
            labels.append(label_type)
        train_data.append(output_data)

        # 输出theta ner训练数据格式
        # with open(Path(data_dir, "train_data.txt"), 'w', encoding='utf-8') as wt:
        with open(Path(data_dir, train_dir + "_data.txt"), 'w', encoding='utf-8') as wt:
            for output_data in train_data:
                output_string = json.dumps(output_data, ensure_ascii=False)
                wt.write(f"{output_string}\n")


# 为不影响label的生成，需先dev后train
conver_labels('train')
print(Counter(labels))

with open(Path(data_dir, "labels.txt"), 'w', encoding='utf-8') as wt:
    labels = list(set(labels))
    for output_string in labels:
        wt.write(f"{output_string}\n")

# 转换测试数据
with open(Path(data_dir, "test_data.txt"), 'w', encoding='utf-8') as wt:
    for file in tqdm(list(Path(data_dir, 'test').glob('*.txt'))[:]):
        originalText = load_txt(file)
        # originalText = originalText.replace(' ',',')
        if originalText[:1] == ' ': originalText = '@' + originalText[1:]
        txt_list.append(originalText)
        file_name = os.path.basename(file).split('.')[0]
        output_data = {'originalText': originalText, 'uid': file_name}
        output_string = json.dumps(output_data, ensure_ascii=False)
        wt.write(f"{output_string}\n")

txt_list = ['【' + str(len(txt)) + '】' for txt in txt_list]
count = Counter(txt_list)
print(count)




#!/usr/bin/env python
# coding:utf-8
''' 
@attention: ssh客户端使用 
'''
import paramiko
from paramiko.py3compat import u
import time
class SSHClient(object): 
    def __init__(self): 
        self.ssh = None
    
    '''
    @attention: 关闭 ssh 链接 
    @param ssh: ssh链接
    '''
    def close(self):
        self.ssh.close()
    '''
    @attention: 创建 ssh 链接 
    @param v_username: 用户名
    @param v_password: 密码
    @param v_ip: IP
    @param v_port: 端口号
    '''
    def sshConnection(self, v_ip, v_username, v_password,  v_port=22):
        # 创建SSH对象
        ssh = paramiko.SSHClient()
        # 把要连接的机器添加到known_hosts文件中
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        # 连接服务器
        ssh.connect(hostname=v_ip, port=v_port, username=v_username, password=v_password)
        self.ssh = ssh
        return self
    # endregion
    '''
    @attention: 执行单条命令 
    @param ssh: ssh链接
    @param v_cmd: 需要执行的命令
    '''
    def sshExecByOne(self, v_cmd): 
        assert self.ssh,'ssh is None ,set sshConnection now'
        # 执行
        stdin, stdout, stderr = self.ssh.exec_command(v_cmd)
        result = stdout.read()
         
        if not result:
            result = stderr.read()
        return result.decode()
    '''
    @attention: 执行命令集
    @author: 白瑞钧
    @param s: ssh链接
    @param l_cmd: 需要执行的命令集
    @param exec_wait: 执行命令间隔时间
    @param exit_wait: 退出等待时间
    '''
    def sshExecByMany(self,l_cmd, exec_wait=1, exit_wait=1):
        assert self.ssh,'ssh is None ,set sshConnection now'
        
        ssh = self.ssh.invoke_shell()
        # 执行
        for v_cmd in l_cmd:
            ssh.send(v_cmd)
            ssh.send('\n')
            time.sleep(exec_wait)
            if v_cmd=='exit':
                time.sleep(exit_wait)
        result = u(ssh.recv(9999))
        return result
    

if __name__ == '__main__':
    SSH = SSHClient().sshConnection('192.168.54.22','root', 'yirong123',10022) 
    l_cmd = ['docker ps -a']
    result = SSH.sshExecByMany(l_cmd, 1, 1)
    print(result)
    SSH.close()
    # getClient = SSHClient()
    # ssh = getClient.sshConnection('sys_admin', 'XSW@1qaz', '10.82.28.219')
    # result = getClient.sshExecByOne(ssh,'pwd')
    # print(result)
    # getClient.close(ssh)
''' '''


# -*- coding: utf-8 -*-
import re
from itertools import groupby

from ..reg import RegularExtrator_txt as pre

class RegularExtrator(pre):
 
    def __init__(self, text='',data_list = []):
        pre.__init__(self, text,data_list)  
        self.text = text
        self.data_list = data_list

    def remove_chinese(self):
        if self.text == "":
            return ""
        pattern = re.compile(u'[\u4E00-\u9FA5]')
        text_without_chinese = pattern.sub(r"", self.text)
        return text_without_chinese 
    
    #email抽取
    def extract_email(self):
        if self.text == "":
            return []
        text_without_chinese = self.remove_chinese()
        text_without_chinese = text_without_chinese.replace(' at ', '@').replace(' dot ', '.')
        sep = ',!?:; ，。！？《》、|\\/'
        eng_split_texts = [''.join(g) for k, g in groupby(text_without_chinese, sep.__contains__) if not k]
        email_pattern = r'^[a-zA-Z0-9_-]+@[a-zA-Z0-9_-]+(\.[a-zA-Z_-]+)+$'
        emails = []
        for eng_text in eng_split_texts:
            result = re.match(email_pattern, eng_text, flags=0)
            if result:
                emails.append(result.string)
        return emails

    #电话号码抽取
    def extract_chinese_cellphone(self):
        if self.text == "":
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if len(ele) >= 7 and len(ele) < 17]
        phone_pattern = r'^((\+86)?([- ])?)?(|(13[0-9])|(14[0-9])|(15[0-9])|(17[0-9])|(18[0-9])|(19[0-9]))([- ])?\d{3}([- ])?\d{4}([- ])?\d{4}$'
        phones = []
        for eng_text in eng_split_texts_clean:
            result = re.match(phone_pattern, eng_text, flags=0)
            if result:
                phones.append(result.string.replace('+86', '').replace('-', ''))
        return phones

    #身份证抽取
    def extract_indentity_ids(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if len(ele) == 18]
        id_pattern = r'^[1-9][0-7]\d{4}((19\d{2}(0[13-9]|1[012])(0[1-9]|[12]\d|30))|(19\d{2}(0[13578]|1[02])31)|(19\d{2}02(0[1-9]|1\d|2[0-8]))|(19([13579][26]|[2468][048]|0[48])0229))\d{3}(\d|X|x)?$'
        phones = []
        for eng_text in eng_split_texts_clean:
            result = re.match(id_pattern, eng_text, flags=0)
            if result:
                phones.append(result.string.replace('+86', '').replace('-', ''))
        return phones

    #ipv4地址抽取
    def extract_ipv4_addresses(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if len(ele) == 32]
        ip_pattern = r'^(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)\.(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)\.(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)\.(25[0-5]|2[0-4]\d|[0-1]\d{2}|[1-9]?\d)$'
        ip_addresses = []
        for eng_text in eng_split_texts_clean:
            result = re.match(ip_pattern, eng_text, flags=0)
            if result:
                ip_addresses.append(result.string)
        return ip_addresses

    #QQ号码抽取
    def extract_QQ_number(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if len(ele) >= 5 and len(ele) <= 10]
        QQ_pattern = r'^[1-9]([0-9]{5,11})$'
        QQ_numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(QQ_pattern, eng_text, flags=0)
            if result:
                QQ_numbers.append(result.string)
        return QQ_numbers

    #国内固话抽取
    def extract_china_fixed_number(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 7 or len(ele) == 8 or len(ele) == 11)]
        fixed_number_pattern = r'^[0-9-()（）]{7,18}$'
        fixed_numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(fixed_number_pattern, eng_text, flags=0)
            if result:
                fixed_numbers.append(result.string)
        return fixed_numbers

    #国内移动电话号码抽取
    def extract_china_mobile_phone_number(self, pattern):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        if pattern == "all":
            pattern = r'^(?:\+?86)?1(?:3\d{3}|5[^4\D]\d{2}|8\d{3}|7(?:[35678]\d{2}|4(?:0\d|1[0-2]|9\d))|9[01356789]\d{2}|66\d{2})\d{6}$'
        elif pattern == "china_mobile":
            pattern = r'^(?:\+?86)?1(?:3(?:4[^9\D]|[5-9]\d)|5[^3-6\D]\d|8[23478]\d|(?:78|98)\d)\d{7}$'
        elif pattern == "china_unicom":
            pattern = r'^(?:\+?86)?1(?:3[0-2]|[578][56]|66)\d{8}$'
        elif pattern == "china_telecom":
            pattern = r'^(?:\+?86)?1(?:3(?:3\d|49)\d|53\d{2}|8[019]\d{2}|7(?:[37]\d{2}|40[0-5])|9[139]\d{2})\d{6}$'
        elif pattern == "inmarsat":#国际海事卫星电话
            pattern = r'^(?:\+?86)?1749\d{7}$'
        elif pattern == "emergency_communications":#紧急电话
            pattern = r'^(?:\+?86)?174(?:0[6-9]|1[0-2])\d{6}$'
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers

    #虚拟号码
    def extract_china_mvno_number(self, pattern):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        if pattern == "all":
            pattern = r'^(?:\+?86)?1(?:7[01]|6[257])\d{8}$'
        elif pattern == "china_mobile":
            pattern = r'^(?:\+?86)?1(?:65\d|70[356])\d{7}$'
        elif pattern == "china_unicom":
            pattern = r'^(?:\+?86)?1(?:70[4789]|71\d|67\d)\d{7}$'
        elif pattern == "china_telecom":
            pattern = r'^(?:\+?86)?1(?:70[012]|62\d)\d{7}$'
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers

    #物联网号码
    def extract_china_IoT_number(self, pattern):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        if pattern == "all":
            pattern = r'^(?:\+?86)?14(?:[14]0\d|[68]\d{2})\d{8}$'
        elif pattern == "china_mobile":
            pattern = r'^(?:\+?86)?14(?:40|8\d)\d{9}$'
        elif pattern == "china_unicom":
            pattern = r'^(?:\+?86)?146\d{10}$'
        elif pattern == "china_telecom":
            pattern = r'^(?:\+?86)?1410\d{9}$'
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers

    #上网卡
    def extract_china_data_only_number(self, pattern):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        if pattern == "all":
            pattern = r'^(?:\+?86)?14[579]\d{8}$'
        elif pattern == "china_mobile":
            pattern = r'^(?:\+?86)?147\d{8}$'
        elif pattern == "china_unicom":
            pattern = r'^(?:\+?86)?145\d{8}$'
        elif pattern == "china_telecom":
            pattern = r'^(?:\+?86)?149\d{8}$'
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers

    #所有号码
    def extract_all_china_phone_number(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        pattern = "^(?:\+?86)?1(?:3\d{3}|5[^4\D]\d{2}|8\d{3}|7(?:[01356789]\d{2}|4(?:0\d|1[0-2]|9\d))|9[01356789]\d{2}|6[2567]\d{2}|4(?:[14]0\d{3}|[68]\d{4}|[579]\d{2}))\d{6}$"
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers

    #所有支持短信功能的号码
    def extract_all_china_phone_number_support_message(self):
        if self.text == '':
            return []
        eng_texts = self.remove_chinese()
        sep = ',!?:; ：，.。！？《》、|\\/abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
        eng_split_texts = [''.join(g) for k, g in groupby(eng_texts, sep.__contains__) if not k]
        eng_split_texts_clean = [ele for ele in eng_split_texts if (len(ele) == 11 or len(ele) == 13)]
        pattern = "^(?:\+?86)?1(?:3\d{3}|5[^4\D]\d{2}|8\d{3}|7(?:[01356789]\d{2}|4(?:0\d|1[0-2]|9\d))|9[01356789]\d{2}|6[2567]\d{2}|4[579]\d{2})\d{6}$"
        numbers = []
        for eng_text in eng_split_texts_clean:
            result = re.match(pattern, eng_text, flags=0)
            if result:
                numbers.append(result.string.replace('+86', '').replace('-', ''))
        return numbers


    def extract_date(self):
        '''正则表达式提取文本所有日期
    
        :param text: 待检索文本
    
               TODO 待优化
        >>> match_date('日期是2020-05-20 13:14:15.477062.')
        ['2020-05-20']
        '''
        if self.text == '':
            return []
        text = self.text.replace("年", "-").replace("月", "-").replace("日", " ").replace("号", " ").replace("/", "-").strip()
        pattern = r'(\d{4}-\d{1,2}-\d{1,2})'
        pattern = re.compile(pattern)
        result = pattern.findall(text)
        return result
    
    
    def extract_datetime(self):
        '''正则表达式提取文本所有日期+时间
               TODO 待优化
        :param text: 待检索文本
    
        >>> match_datetime('日期是2020-05-20 13:14:15.477062.')
        ['2020-05-20 13:14:15']
        '''
        if self.text == '':
            return []
        text = self.text.replace("年", "-").replace("月", "-").replace("日", " ").replace("号", " ").replace("时", ":").replace("分", ":").replace("秒", " ").replace("/", "-").strip()
        pattern = r'(\d{4}-\d{1,2}-\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2})'
        pattern = re.compile(pattern)
        result = pattern.findall(text)
        return result




from pydantic import BaseModel
import uvicorn  ,time
from fastapi import Query,UploadFile,File,BackgroundTasks
from opspipe.app.settings.config import DEBUG
from opspipe.app.main import get_application,get_para
from opspipe.app.core.response.BaseResponse import success, fail
from opspipe.utils import utils
from opspipe.base import logger
app = get_application()
app.description =  'test background task'


class Item(BaseModel):  # 创建数据模型,将你的数据模型声明为继承自 BaseModel 的类
    text: str =  Query(  
        '',
        description="我是请求参数",
        deprecated=True 
        )

@app.post("/test",tags=["/test"],
          summary="我是接口描述",
          description='json格式以post方式提交')
async def test(item: Item,  background_tasks: BackgroundTasks): 
    text = item.dict()['text']
    args = background_tasks.is_async
    background_tasks.add_task(test1,text) 
    return success(text)
 
def test1(txt):
    for i in range(1000):
        print(txt)
        time.sleep(5)
        


if __name__ == '__main__':
    host , port = get_para()
    uvicorn.run(app='__main__:app', host=host, port=port, reload=True, debug=DEBUG)



# encoding: utf-8
'''
import datetime
import logging
import os
from loguru import logger

from ...settings.config import LOG_DIR 

try:
    log_level = os.environ['LOG_LEVEL']
except KeyError:
    log_level = 'INFO'  # pylint: disable=invalid-name (C0103)
 
today = datetime.datetime.now().strftime("%Y-%m-%d") 
log_file = os.path.join(LOG_DIR, "runtime-" + today + ".log") 

logger.add(log_file, encoding='utf-8', enqueue=True)
'''
    
     



#!/usr/bin/env python
# -*- coding: utf-8 -*- 
import os, json,requests 
import uvicorn  
from opspipe.app.settings.config import DEBUG
from opspipe.app.main import get_application,get_para
from opspipe.app.core.response.BaseResponse import success, fail 
from fastapi import Query  
from loguru import logger   
from pydantic import BaseModel
from fastapi import UploadFile, File
from opspipe.base.logger import logtime
from opspipe.utils import utils
from construction_enterprise_qualification_certificate import get_result
# -------------------- System --------------------   
app = get_application()
app.description =  '建筑业企业资质证书识别'
 
 
@logtime
def ocr(path):
    url = "http://192.168.55.199:30007/icr/recognize_document" 
    files = {'filename': open(path, 'rb')}
    r = requests.post(url, files=files) 
    dt = json.loads(r.text) 
    linesText = dt['linesText']
    return '\n'.join(linesText) ,r.text
 
class Item(BaseModel):
    text: str = Query(
        '',
        description="待抽取文本",
        deprecated=True
    )
 

@app.post("/predict_img", summary="图片抽取接口",)
async def predict_img(file: UploadFile = File(...)):  
    img_path = utils.save_file(file)
    try:     
        txt,json = ocr(img_path) 
        print(txt)
        result = get_result(txt) 
        return success(result)
    
    except Exception as err: 
        logger.error(f"{err}")
        # TOTO 优化500 全局错误 
        return fail(400)
    
@app.post("/predict",summary="纯文本抽取接口")
async def predict(item: Item):
    text = item.dict()['text']
    result = get_result(text)
    return success(result)
  
     
if __name__ == '__main__':
    host , port = get_para()
    uvicorn.run(app='__main__:app', host=host, port=port, reload=True, debug=DEBUG)



"""
路由
""" 
from fastapi import APIRouter
from fastapi.responses import HTMLResponse 
from ...settings import config 
from starlette.responses import RedirectResponse  
router = APIRouter()

@router.get("/", summary="首页", include_in_schema=False)
async def root():
    return RedirectResponse('/docs')


@router.get(config.MDDOC_URL, summary="生成 .MD 接口文档", response_class=HTMLResponse)
async def mddoc():   
    with open(config.MDDOC_DIR, 'r', encoding='utf-8') as f: 
        mddoc = f.read()
        return mddoc



# -*- coding: utf-8 -*-
'''
Created on 2020-12-6

@author: zhys513(254851907@qq.com)
'''
from opspipe.ops.deploy.docker_client import DockerClient 
from opspipe.ops.deploy.ssh_client import SSHClient 


SSH = SSHClient().sshConnection('192.168.54.22','root', 'yirong123',10022)
DOCKER = DockerClient(SSH)
result = DOCKER.build('test111','/root/pack_tmp/aip')
'''
result = DOCKER.stop('ai_special_1.0')
print(result)
result = DOCKER.start('ai_special_1.0')
print(result)
'''
#result = DOCKER.logs('ai_special_1.0',10)
print(result)


# -*- coding: utf-8 -*-
'''
Created on 2020-9-24
先到标注工具新建类型，导入json，再导出json作为model.json
(TODO 标注工具还无法实现导入，无法固定模板)

@author: zhys513(254851907@qq.com)
'''
import json,os 
from tqdm import tqdm
from pathlib import Path
from theta.utils import load_json_file
from loguru import logger
from ...utils import read_json
from ..data_generator import train_data_generator

def theta_train2poplar(args,train_file = 'train_data.txt'):
    '''theta 训练数据 转 标注
    强绑定theta
    '''
    datasets_path = Path(args.data_dir,args.dataset_name)
    train_file = Path(datasets_path,train_file)
    theta2poplar(datasets_path,train_file)

def theta_submission2poplar(args,submission_file = ''):
    '''theta 结果数据 转 标注
    强绑定theta
    '''
    datasets_path = Path(args.data_dir,args.dataset_name)
    if submission_file :
        submission_file = f"{args.submissions_dir}/{args.dataset_name}_submission_{args.local_id}.json.txt"
    theta2poplar(datasets_path,submission_file)

def theta2poplar(datasets_path,json_file,
                 output_path='poplar_output',
                 model_file='model.json'):
    '''theta 数据 转 标注  model_file 为必传项
    可配置'''
    poplar_output = os.path.join(datasets_path,output_path)
    if not os.path.exists(poplar_output):
        os.mkdir(poplar_output)

    # 标注模板 从模板中定义ID和颜色
    label_modle = read_json(Path(datasets_path, model_file))
    label2id, id2label, id2label_color, categorie2id = {}, {}, {}, {}
    for labelCategorie in label_modle['labelCategories']:
        label2id[labelCategorie['text']] = labelCategorie['id']
        id2label[labelCategorie['id']] = labelCategorie['text']
        id2label_color[labelCategorie['id']] = labelCategorie['color']
    for connectionCategorie in label_modle['connectionCategories']:
        categorie2id[connectionCategorie['text']] = connectionCategorie['id']

    # 获取 theta ner 数据格式
    train_datas = load_json_file(json_file)

    for i, x in enumerate(tqdm(train_datas)):
        guid = str((i + 1) * 10000)
        entities = x['entities']
        originalText = x['originalText']
        poplar_labels = []
        for n, entity in enumerate(entities):
            eid = guid + str(n)
            start_pos = entity['start_pos']
            end_pos = entity['end_pos'] - 1
            category = entity['label_type']
            mention = originalText[start_pos:end_pos]
            entity = {
                "id": eid,
                "categoryId": label2id[category],
                "startIndex": start_pos,
                "endIndex": end_pos + 1,
                "mention":mention
            }
            poplar_labels.append(entity)

        poplar_json = label_modle.copy()
        poplar_json['id'] = guid
        poplar_json['title'] = i
        poplar_json['content'] = originalText
        poplar_json['labels'] = poplar_labels

        with open(Path(poplar_output, guid + '.json'), 'w', encoding='utf-8') as wt:
            output_string = json.dumps(poplar_json, ensure_ascii=False, indent=4)
            wt.write(f"{output_string}\n")

def to_poplar(args,ner_labels):
    '''theta 自带'''
    start_pages = 10
    max_pages = 10
    poplar_json = {
        "content": "",
        "labelCategories": [],
        "labels": [],
        "connectionCategories": [],
        "connections": []
    }

    poplar_colorset = [
        '#007bff', '#17a2b8', '#28a745', '#fd7e14', '#e83e8c', '#dc3545',
        '#20c997', '#ffc107', '#007bff'
    ]
    label2id = {x: i for i, x in enumerate(ner_labels)}
    label_categories = poplar_json['labelCategories']
    for _id, x in enumerate(ner_labels):
        label_categories.append({
            "id": _id,
            "text": x,
            "color": poplar_colorset[label2id[x]],
            "borderColor": "#cccccc"
        })

    poplar_labels = poplar_json['labels']
    poplar_content = ""
    eid = 0
    num_pages = 0
    page_offset = 0
    for guid, text, _, entities in train_data_generator(args.train_file,ner_labels):
        if num_pages < start_pages:
            num_pages += 1
            continue

        page_head = f"\n-------------------- {guid} Begin --------------------\n\n"
        page_tail = f"\n-------------------- {guid} End --------------------\n\n"
        poplar_content += page_head + f"{text}" + page_tail

        for entity in entities:
            poplar_labels.append({
                "id":
                    eid,
                "categoryId":
                    label2id[entity.category],
                "startIndex":
                    page_offset + len(page_head) + entity.start,
                "endIndex":
                    page_offset + len(page_head) + entity.end + 1,
            })
            eid += 1

        num_pages += 1
        if num_pages - start_pages >= max_pages:
            break

        page_offset = len(poplar_content)

    poplar_json["content"] = poplar_content
    poplar_json['labels'] = poplar_labels

    poplar_data_file = Path(args.local_dir,'datasets','poplar_data.json')
    json.dump(poplar_json,
              open(poplar_data_file, 'w'),
              ensure_ascii=False,
              indent=2)
    logger.info(f"Saved {poplar_data_file}")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
from theta.utils import load_json_file,seg_generator
from tqdm import tqdm
import random,os
from pathlib import Path
from  ...utils import clean_text

def theta2bio(data_root,output_dir,maxlen=512,seg_backoff=128,train_rate=0.9,
              theta_train_file = 'train_data.txt',
              bio_train_file='bio_data.train',bio_dev_file='bio_data.dev'):
    train_file = os.path.join(data_root, theta_train_file)
    train_txts = load_json_file(train_file)
    # lines = train_txts + val_txts

    train = open(Path(output_dir, bio_train_file), 'w', encoding='UTF-8')
    val = open(Path(output_dir, bio_dev_file), 'w', encoding='UTF-8')
    train_txts, val_txts = split_datas(train_txts, True,train_rate)
    create_data(train_txts, train,maxlen,seg_backoff)
    create_data(val_txts, val,maxlen,seg_backoff)

    train.close()
    val.close()


def split_datas(full_list,shuffle=False,train_rate=0.9):
    n_total = len(full_list)
    offset = int(n_total * train_rate)
    if n_total==0 or offset<1:
        return [],full_list
    if shuffle:
        random.shuffle(full_list)
        sublist_1 = full_list[:offset]
        sublist_2 = full_list[offset:]
        return sublist_1,sublist_2

def create_data(lines, w,maxlen=512,seg_backoff=128):
    for i, x in enumerate(tqdm(lines)):
        # guid = str(i)
        text = clean_text(x['originalText'])
        entities = x['entities']
        labels = []
        for entity_json in entities:
            start_pos = entity_json['start_pos']
            end_pos = entity_json['end_pos']
            category = entity_json['label_type']
            # 矫正前后带空格
            # entity = text[start_pos:end_pos]
            labels.append([start_pos, end_pos, category])

        g = seg_generator((text,), maxlen - 2, seg_backoff)
        for t in g:
            text = t[0][0]
            num = t[1]
            stripped_txt = [['O'] * 2 for _ in range(len(text))]
            for i, s in enumerate(text):
                stripped_txt[i][0] = s

            for lable in labels:
                start = lable[0] - num
                end = lable[1] - num
                if start < 0 or end < 0 or start > len(text) or end > len(text):
                    continue

                is_B = True
                for i in range(start, end):
                    if is_B:
                        stripped_txt[i][1] = 'B-' + lable[2]
                        is_B = False
                    else:
                        stripped_txt[i][1] = 'I-' + lable[2]

        for st in stripped_txt:
            output_string = st[0] + ' ' + st[1]
            w.write(f"{output_string}\n")
        w.write(f"\n")


# -*- coding: utf-8 -*-

from pycocotools.coco import COCO
import random, cv2 
import numpy as np

class Base():
    def __init__(self,images=[],annotations = [],categories = []): 
        self.images = images 
        self.annotations = annotations
        self.categories = categories 
        self.splitdatas = []
    
    '''
        读取原始数据
    '''
    def load(self,path): 
        # TODO categories 转换 
        raise NotImplementedError
         
         
    '''
        将名字和id号建立一个字典
    '''
    def name2catid(self):#
        classes = dict()
        for cat in self.categories:
            name = cat['name']
            if isinstance(name,list):
                name = cat['name'][0]
            classes[name] = cat['id'] 
        return classes
     
    '''
        将名字和id号建立一个字典
    '''
    def catid2name(self):
        classes = dict()
        for cat in self.categories:
            name = cat['name']
            if isinstance(name,list):
                name = cat['name'][0]
            classes[cat['id']] = name 
        return classes

    '''
        将一个数据转换为另一个数据
    '''
    def converfrom(self,fun):
        self.images = fun.images
        self.annotations = fun.annotations
        self.categories = fun.categories 
    
    
    '''
        切分数据,默认7:1:2 切分
    '''
    def split(self,seed=102,division_ratio=[7,1,2]): 
        random.seed(seed)
        coco = self.getcoco()
        imgIds = coco.getImgIds()
        num = len(imgIds) 
        px = self.get_percent(division_ratio)
        splitdatas = []
        for percent in px: 
            data_num = int(num*percent)  
            iids = random.sample(imgIds,data_num) 
            images = coco.loadImgs(iids)
            annotations = coco.loadAnns(iids)
            newcoco = self.getcoco(images,annotations) 
            splitdatas.append(newcoco)
        self.splitdatas = splitdatas
        return splitdatas
    
    '''
        数据生成
    '''
    def generator(self,save_path):
        raise NotImplementedError 
    
    '''
        数据分析
    '''
    def eda(self,savepath,show = False):
        raise NotImplementedError 
    
    
    def getcoco(self,images=None,annotations=None,categories=None): 
        if not images :
            images = self.images
        if not annotations :
            annotations = self.annotations
        if not categories :
            categories = self.categories
            
        dataset = {"images": images, "type": "instances", "annotations": annotations, "categories": categories} 
        coco = COCO() 
        coco.dataset = dataset
        coco.createIndex()
        return coco

    # 数据比例分割
    def get_percent(self,division_ratio):
        ps = []
        p = 0
        for i in division_ratio:
            p = p + i
        for i in division_ratio:
            px = i / p
            ps.append(px) 
        
        return ps


    def imread(self,img_path):
        img = cv2.imdecode(np.fromfile(img_path,dtype=np.uint8),-1)
        '''
        打印异常图片
        try:
            img.shape[2]
        except:
            print(img_path)
        '''
        if len(img.shape) != 3:
            #img = img.unsqueeze(0)
            image  = np.expand_dims(img, axis=2)
            img = np.concatenate((image , image , image ), axis=-1)
        return img
    
class ImageInfo():
    def __init__(self,iid,file_name,height,width): 
        self.id = iid
        self.file_name = file_name
        self.height = height
        self.width = width

class CategorieInfo():
    def __init__(self,cid,name,supercategory=None): 
        self.id = cid
        self.name = name
        self.supercategory = supercategory
        
     
class AnnotationInfo():
    def __init__(self,aid,image_id,category_id,bbox=[],area='',iscrowd = 0,ignore=None,segmentation=[]):  
        self.id = aid
        self.image_id = image_id
        self.category_id = category_id
        self.bbox = bbox
        self.area = area
        self.iscrowd = iscrowd
        self.ignore = ignore
        self.segmentation = segmentation
        


# -*- coding: utf-8 -*-
class Base():
    def __init__(self, data = None): 
        self.data = data
    
    def read(self,path): 
        raise NotImplementedError
        
    def converfrom(self,fun):
        raise NotImplementedError
    
    def save(self,path):
        raise NotImplementedError


"""
    参数异常返回封装
"""
from typing import Union
from fastapi.exceptions import RequestValidationError
from pydantic import ValidationError
from fastapi import Request, status
from fastapi.responses import JSONResponse


async def http422_error_handler(_: Request, exc: Union[RequestValidationError, ValidationError],) -> JSONResponse:
    return JSONResponse(
        {
            "code": status.HTTP_422_UNPROCESSABLE_ENTITY,
            "message": "参数校验错误",
            "data": exc.errors(),
        },
        # status_code=status.HTTP_422_UNPROCESSABLE_ENTITY,
        status_code=200,
    )


# -*- coding: utf-8 -*-
'''
数据生成器
'''
import json
from tqdm import tqdm
from pathlib import Path
from loguru import logger
from theta.utils import load_json_file, split_train_eval_examples
from theta.modeling import LabeledText, load_ner_examples, load_ner_labeled_examples

def train_data_generator(args):
    lines = load_json_file(args.train_file)

    for i, x in enumerate(tqdm(lines)):
        guid = str(i)
        text = x['originalText']
        sl = LabeledText(guid, text)

        entities = x['entities']
        for entity in entities:
            start_pos = entity['start_pos']
            end_pos = entity['end_pos'] - 1
            category = entity['label_type']
            mention = entity['mention']

            # 矫正前后带空格
            ent = text[start_pos:end_pos + 1]
            if mention != ent:
                print('###', mention, ent)
            if len(ent.lstrip()) != len(ent):
                l = len(ent) - len(ent.lstrip())
                start_pos = start_pos + l
                logger.warning(f'{ent} --lstrip-- {text[start_pos:end_pos + 1]}')

            if len(ent.rstrip()) != len(ent):
                l = len(ent) - len(ent.rstrip())
                end_pos = end_pos - l
                logger.warning(f'{ent} --rstrip-- {text[start_pos:end_pos + 1]}')

            if category not in args.ner_labels:
                continue

            sl.add_entity(category, start_pos, end_pos)

        yield str(i), text, None, sl.entities


'''
def load_train_val_examples(args):

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})



    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples 
'''

def load_train_val_examples(args):
    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    train_base_examples = load_ner_labeled_examples(
        lines,
        args.ner_labels,
        seg_len=args.seg_len,
        seg_backoff=args.seg_backoff,
        num_augements=args.num_augements,
        allow_overlap=allow_overlap)

    train_examples, val_examples = train_base_examples, []
    # 如果训练集和验证集一样就自动切分
    if args.train_file == args.eval_file:
        train_examples, val_examples = split_train_eval_examples(
            train_base_examples,
            train_rate=args.train_rate,
            fold=args.fold,
            shuffle=False)
    else:
        lines = []
        for guid, text, _, entities in train_data_generator(args.eval_file):
            sl = LabeledText(guid, text, entities)
            lines.append({'guid': guid, 'text': text, 'entities': entities})

        val_examples = load_ner_labeled_examples(
            lines,
            args.ner_labels,
            seg_len=args.seg_len,
            seg_backoff=args.seg_backoff,
            num_augements=args.num_augements,
            allow_overlap=allow_overlap)

    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples


def test_data_generator(test_file):
    lines = load_json_file(test_file)
    for i, s in enumerate(tqdm(lines)):
        i = s['uid']
        guid = str(i)
        text_a = s['originalText']

        yield guid, text_a, None, None


def load_test_examples(args):
    test_base_examples = load_ner_examples(test_data_generator,
                                           args.test_file,
                                           seg_len=args.seg_len,
                                           seg_backoff=args.seg_backoff)

    logger.info(f"Loaded {len(test_base_examples)} test examples.")
    return test_base_examples

def generate_submission(args):
    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
    reviews = json.load(open(reviews_file, 'r'))

    submission_file = f"{args.submissions_dir}/{args.dataset_name}_submission_{args.local_id}.json.txt"
    with open(submission_file, 'w') as wt:
        for guid, json_data in reviews.items():
            output_data = {'originalText': json_data['text'], 'entities': []}
            for json_entity in json_data['entities']:
                output_data['entities'].append({
                    'label_type':
                        json_entity['category'],
                    'overlap':
                        0,
                    'start_pos':
                        json_entity['start'],
                    'end_pos':
                        json_entity['end'] + 1,
                    'mention':
                        json_entity['mention']
                })
            output_data['entities'] = sorted(output_data['entities'],
                                             key=lambda x: x['start_pos'])
            output_string = json.dumps(output_data, ensure_ascii=False)
            wt.write(f"{output_string}\n")

    logger.info(f"Saved {len(reviews)} lines in {submission_file}")

def to_poplar(args):
    start_pages = 10
    max_pages = 10
    poplar_json = {
        "content": "",
        "labelCategories": [],
        "labels": [],
        "connectionCategories": [],
        "connections": []
    }

    poplar_colorset = [
        '#007bff', '#17a2b8', '#28a745', '#fd7e14', '#e83e8c', '#dc3545',
        '#20c997', '#ffc107', '#007bff'
    ]
    label2id = {x: i for i, x in enumerate(args.ner_labels)}
    label_categories = poplar_json['labelCategories']
    for _id, x in enumerate(args.ner_labels):
        label_categories.append({
            "id": _id,
            "text": x,
            "color": poplar_colorset[label2id[x]],
            "borderColor": "#cccccc"
        })

    poplar_labels = poplar_json['labels']
    poplar_content = ""
    eid = 0
    num_pages = 0
    page_offset = 0
    for guid, text, _, entities in train_data_generator(args.train_file):
        if num_pages < start_pages:
            num_pages += 1
            continue

        page_head = f"\n-------------------- {guid} Begin --------------------\n\n"
        page_tail = f"\n-------------------- {guid} End --------------------\n\n"
        poplar_content += page_head + f"{text}" + page_tail

        for entity in entities:
            poplar_labels.append({
                "id":
                    eid,
                "categoryId":
                    label2id[entity.category],
                "startIndex":
                    page_offset + len(page_head) + entity.start,
                "endIndex":
                    page_offset + len(page_head) + entity.end + 1,
            })
            eid += 1

        num_pages += 1
        if num_pages - start_pages >= max_pages:
            break

        page_offset = len(poplar_content)

    poplar_json["content"] = poplar_content
    poplar_json['labels'] = poplar_labels

    poplar_data_file = Path(args.local_dir,'datasets','poplar_data.json')
    json.dump(poplar_json,
              open(poplar_data_file, 'w'),
              ensure_ascii=False,
              indent=2)
    logger.info(f"Saved {poplar_data_file}")



'''
class NER_CV():
    def __init__(self, h=None, lam=1,maxiter=500, tol=1e-6):
        self.beta = None
        self.h = h
        self.dataset = None
        self.maxiter = maxiter
        self.tol = tol
        self.funvalue = None
        self.coef = None
        self.dataset = None
        self.lam=lam
        self.iteration=maxiter


    def fit(self, X_train, y_train):
        #用于训练模型参数，例如self.coef
        self.coef, self.funvalue = myfun(X_train, y_train)

    def predict(self, X_new):
        #用于根据X预测y，返回y的预测值数组
        #XXXXXXX
        return y_pre


    def get_params(self, deep=True):
        """Get parameters for this estimator.

        Parameters
        ----------
        deep : boolean, optional
            If True, will return the parameters for this estimator and
            contained subobjects that are estimators.

        Returns
        -------
        params : mapping of string to any
            Parameter names mapped to their values.
        """
        out = dict()
        for key in ['h','lam','maxiter','tol']:#这里是所用超参数的list
            value = getattr(self, key, None)
            if deep and hasattr(value, 'get_params'):
                deep_items = value.get_params().items()
                out.update((key + '__' + k, val) for k, val in deep_items)
            out[key] = value
        return out

    def set_params(self, **params):
        """Set the parameters of this estimator.

        The method works on simple estimators as well as on nested objects
        (such as pipelines). The latter have parameters of the form
        ``<component>__<parameter>`` so that it's possible to update each
        component of a nested object.

        Returns
        -------
        self
        """
        if not params:
            # Simple optimization to gain speed (inspect is slow)
            return self
        valid_params = self.get_params(deep=True)


        for key, value in params.items():
            if key not in valid_params:
                raise ValueError('Invalid parameter %s for estimator %s. '
                                 'Check the list of available parameters '
                                 'with `estimator.get_params().keys()`.' %
                                 (key, self))
            setattr(self, key, value)
            valid_params[key] = value

        return self

    def score(self, X, y, sample_weight=None):
    	#如果这里不设置score函数，可以在GridSearchCV()的scoring参数中指定
        """Returns the mean accuracy on the given test data and labels.

        In multi-label classification, this is the subset accuracy
        which is a harsh metric since you require for each sample that
        each label set be correctly predicted.

        Parameters
        ----------
        X : array-like, shape = (n_samples, n_features)
            Test samples.

        y : array-like, shape = (n_samples) or (n_samples, n_outputs)
            True labels for X.

        sample_weight : array-like, shape = [n_samples], optional
            Sample weights.

        Returns
        -------
        score : float
            Mean accuracy of self.predict(X) wrt. y.

        """
        return myloss_fun(y, self.predict(X), sample_weight=sample_weight)

'''


# -*- coding: utf-8 -*-
import inspect
import warnings
from functools import partial

from .misc import is_str


class Registry:
    """A registry to map strings to classes.
    Args:
        name (str): Registry name.
    """

    def __init__(self, name):
        self._name = name
        self._module_dict = dict()

    def __len__(self):
        return len(self._module_dict)

    def __contains__(self, key):
        return self.get(key) is not None

    def __repr__(self):
        format_str = self.__class__.__name__ + \
                     f'(name={self._name}, ' \
                     f'items={self._module_dict})'
        return format_str

    @property
    def name(self):
        return self._name

    @property
    def module_dict(self):
        return self._module_dict

    def get(self, key):
        """Get the registry record.
        Args:
            key (str): The class name in string format.
        Returns:
            class: The corresponding class.
        """
        return self._module_dict.get(key, None)

    def _register_module(self, module_class, module_name=None, force=False):
        if not inspect.isclass(module_class):
            raise TypeError('module must be a class, '
                            f'but got {type(module_class)}')

        if module_name is None:
            module_name = module_class.__name__
        if not force and module_name in self._module_dict:
            raise KeyError(f'{module_name} is already registered '
                           f'in {self.name}')
        self._module_dict[module_name] = module_class

    def deprecated_register_module(self, cls=None, force=False):
        warnings.warn(
            'The old API of register_module(module, force=False) '
            'is deprecated and will be removed, please use the new API '
            'register_module(name=None, force=False, module=None) instead.')
        if cls is None:
            return partial(self.deprecated_register_module, force=force)
        self._register_module(cls, force=force)
        return cls

    def register_module(self, name=None, force=False, module=None):
        """Register a module.
        A record will be added to `self._module_dict`, whose key is the class
        name or the specified name, and value is the class itself.
        It can be used as a decorator or a normal function.
        Example:
            >>> backbones = Registry('backbone')
            >>> @backbones.register_module()
            >>> class ResNet:
            >>>     pass
            >>> backbones = Registry('backbone')
            >>> @backbones.register_module(name='mnet')
            >>> class MobileNet:
            >>>     pass
            >>> backbones = Registry('backbone')
            >>> class ResNet:
            >>>     pass
            >>> backbones.register_module(ResNet)
        Args:
            name (str | None): The module name to be registered. If not
                specified, the class name will be used.
            force (bool, optional): Whether to override an existing class with
                the same name. Default: False.
            module (type): Module class to be registered.
        """
        if not isinstance(force, bool):
            raise TypeError(f'force must be a boolean, but got {type(force)}')
        # NOTE: This is a walkaround to be compatible with the old api,
        # while it may introduce unexpected bugs.
        if isinstance(name, type):
            return self.deprecated_register_module(name, force=force)

        # use it as a normal method: x.register_module(module=SomeClass)
        if module is not None:
            self._register_module(
                module_class=module, module_name=name, force=force)
            return module

        # raise the error ahead of time
        if not (name is None or isinstance(name, str)):
            raise TypeError(f'name must be a str, but got {type(name)}')

        # use it as a decorator: @x.register_module()
        def _register(cls):
            self._register_module(
                module_class=cls, module_name=name, force=force)
            return cls

        return _register


def build_from_cfg(cfg, registry, default_args=None):
    """Build a module from config dict.
    Args:
        cfg (dict): Config dict. It should at least contain the key "type".
        registry (:obj:`Registry`): The registry to search the type from.
        default_args (dict, optional): Default initialization arguments.
    Returns:
        object: The constructed object.
    """
    if not isinstance(cfg, dict):
        raise TypeError(f'cfg must be a dict, but got {type(cfg)}')
    if 'type' not in cfg:
        if default_args is None or 'type' not in default_args:
            raise KeyError(
                '`cfg` or `default_args` must contain the key "type", '
                f'but got {cfg}\n{default_args}')
    if not isinstance(registry, Registry):
        raise TypeError('registry must be an mmcv.Registry object, '
                        f'but got {type(registry)}')
    if not (isinstance(default_args, dict) or default_args is None):
        raise TypeError('default_args must be a dict or None, '
                        f'but got {type(default_args)}')

    args = cfg.copy()

    if default_args is not None:
        for name, value in default_args.items():
            args.setdefault(name, value)

    obj_type = args.pop('type')
    if is_str(obj_type):
        obj_cls = registry.get(obj_type)
        if obj_cls is None:
            raise KeyError(
                f'{obj_type} is not in the {registry.name} registry')
    elif inspect.isclass(obj_type):
        obj_cls = obj_type
    else:
        raise TypeError(
            f'type must be a str or valid type, but got {type(obj_type)}')

    return obj_cls(**args)


"""WebSocket"""
# from fastapi.websockets import WebSocket
from starlette.endpoints import WebSocketEndpoint


class Echo(WebSocketEndpoint):
    encoding = "text"

    # 连接
    async def on_connect(self, web_socket):
        await web_socket.accept()

    # 消息
    async def on_receive(self, web_socket, data):
        print(data)
        await web_socket.send_text(f"Message text was: {data}")

    # 断开
    async def on_disconnect(self, web_socket, close_code):
        pass



"""
助手工具
"""
import hashlib
import qrcode
import base64
from io import BytesIO
import uuid
import time


# 密码加密
def en_password(psw: str):
    p = hashlib.md5(psw.encode(encoding='UTF-8')).hexdigest()
    return p


# 密码校验你
def check_password(password: str, old: str):
    np = hashlib.md5(password.encode(encoding='UTF-8')).hexdigest()
    if np == old:
        return True
    else:
        return False


# 字符串转二维码图片二进制
def str_qrcode(txt: str):
    qr = qrcode.QRCode(
        version=5,
        error_correction=qrcode.constants.ERROR_CORRECT_L,
        box_size=4,
        border=4,
    )

    qr.add_data(txt)
    qr.make(fit=True)
    img = qr.make_image()
    return img


# 图片转base64
def img_base64(img: bytes):
    output_buffer = BytesIO()
    img.save(output_buffer, format='PNG')
    binary_data = output_buffer.getvalue()
    base64_data = base64.b64encode(binary_data)
    # base64头
    header = "data:image/png;base64,"
    base64_data = header + str(base64_data.decode('utf-8'))
    return base64_data


# 唯一随机字符串
def only_str():
    only = hashlib.md5(str(uuid.uuid1()).encode(encoding='UTF-8')).hexdigest()
    return str(only)


# 毫秒时间戳
def get_times():
    return int(round(time.time() * 1000))


# 生成token
def create_token(uid: int):
    t = str(uuid.uuid1()) + str(uid)
    return hashlib.md5(t.encode(encoding='UTF-8')).hexdigest()



from pydantic import BaseModel
import uvicorn  
from fastapi import Query,UploadFile,File
from opspipe.app.settings.config import DEBUG
from opspipe.app.main import get_application,get_para
from opspipe.app.core.response.BaseResponse import success, fail
from opspipe.utils import utils
from opspipe.base import logger
app = get_application()
app.description =  '我是API描述'


class Item(BaseModel):  # 创建数据模型,将你的数据模型声明为继承自 BaseModel 的类
    text: str =  Query(  
        '',
        description="我是请求参数",
        deprecated=True 
        )

@app.post("/test",tags=["/test"],
          summary="我是接口描述",
          description='json格式以post方式提交')
async def test(item: Item, ): 
    text = item.dict()['text'] 
    return success(text)


@app.post("/test2")
async def test2(item: str=Query(  
        '',
        description="我是请求参数",
        deprecated=True 
        )): 
    text = item.dict()['text'] 
    return success(text)


@app.post("/test_base64")
async def predict_base64(item: Item):
    #TODO 转换为 文本 
    text = item.dict()['text'] 
    logger.lump_logs('data_file',text) 
    utils.base64_to_file(text,'xxx.txt')
    return success(text)

@app.post("/test_file") 
async def predict_pdf(file: UploadFile = File(...)): 
    pdf_path = utils.save_file(file)
    return success(pdf_path)


@app.post("/test_error") 
async def test_error(): 
    a = ''
    return success(a[1])

if __name__ == '__main__':
    host , port = get_para()
    uvicorn.run(app='__main__:app', host=host, port=port, reload=True, debug=DEBUG)



# -*- coding: utf-8 -*-
'''
Created on 2021-1-28

@author: zhys513(254851907@qq.com)
'''

class GitClient(object): 
    def __init__(self,SSHClient): 
        self.ssh = SSHClient
    
    def login(self,url,usr='',pwd=''):
        pass
        
    def clone(self,url):
        v_cmds = 'git clone ' + url
        result = self.ssh.sshExecByOne(v_cmds) 
        return result
    
    def pull(self,url):
        v_cmds = 'git pull ' + url
        result = self.ssh.sshExecByOne(v_cmds) 
        return result



"""
初始化redis
"""
from fastapi import FastAPI
from aioredis import create_redis_pool, Redis
from ...settings.config import REDIS_HOST, REDIS_PORT, REDIS_DB,REDIS_INIT


async def redis_init(app: FastAPI) -> Redis:
    # Redis.
    # redis_pool = await create_redis_pool(f"redis://{REDIS_HOST}:{REDIS_PORT}/{REDIS_DB}?encoding=utf-8")
    if REDIS_INIT :
        redis_pool = await create_redis_pool(address=f"redis://{REDIS_HOST}:{REDIS_PORT}", db=REDIS_DB, encoding='utf-8')
        app.state.log.info('redis_init ...')
        return redis_pool
    else:
        return None



# -*- coding: utf-8 -*-
'''
Created on 2021-1-10

@author: zhys513(254851907@qq.com)
'''
import os,json
from pycocotools.coco import COCO
from functools import reduce

from opspipe.pipe.cv.data.base import Base

class MsCoco(Base): 
    def __init__(self,images=[],annotations = [],categories = []): 
        Base.__init__(self,images,annotations,categories)  
    
    def load(self,path,data_types=['train'],newId = True):  
        annotations_path = os.path.join(path,'annotations')
        images_path = os.path.join(path,'images')
        # annid重新生成 imgId默认重新生成，根据newId可以配置不重新生成，起始ID为1
        newimgId = 1
        newannId = 1
        # 支持多个数据集进行合并,data_types为空读取annotations下所有数据集
        if data_types:
            pass
        for datatype in data_types:
            annFile = '{}.json'.format(datatype)
            annpath = os.path.join(annotations_path,annFile)
            coco = COCO(annpath) 
            imgIds = coco.getImgIds()
            for imgId in imgIds:
                annIds = coco.getAnnIds(imgIds=[imgId],  iscrowd=None)
                anns = coco.loadAnns(annIds) 
                img = coco.loadImgs(imgId)[0] 
                if newId:
                    imgpath = os.path.join(images_path,datatype)
                    img['id'] = newimgId
                    img['path'] = imgpath
                    newimgId = newimgId + 1
                self.images.append(img)
                for ann in anns:    
                    ann['id'] = newannId
                    newannId = newannId + 1
                    ann['image_id'] = img['id'] 
                    self.annotations.append(ann)
            # cats累加
            self.categories = self.categories + coco.dataset['categories'] 
        # 去重
        self.categories = reduce(lambda x, y: x if y in x else x + [y], [[], ] + self.categories)


    def generator(self,save_path,data_types=['train','val','test']):
        assert len(self.splitdatas) != len(data_types) ,'splitdatas != data_types len'
        for datas in zip(self.splitdatas,data_types):
            print(datas)
        



# -*- coding: utf-8 -*-
'''
Created on 2020-11-11

@author: zhys513(254851907@qq.com)
''' 
# pip3 install camelot-py[cv]
# pip3 install xlsxwriter
import camelot
import pandas as pd

def extract_pdf_table(input_file, output_file):
    """
    只支持text-based 的pdf table
    :param input_file: 输入pdf文件
    :param output_file: 保存每个table，输出到excel文件
    :return:
    """
    #提取pdf的table
    tables = camelot.read_pdf(input_file,flavor='stream')
    table_number = tables.n
    print(f"pdf文件 {input_file} 共包含text-based类型表格 {table_number} 个")

    #准备保存每个table
    writer = pd.ExcelWriter(output_file, engine='xlsxwriter')
    for num in range(table_number):
        table = tables[num]
        # 打印每个table
        print(table.df)
        # 保存到sheet
        table.df.to_excel(writer, sheet_name='page{}_num{}'.format(table.page, num), encoding="utf-8")
    #保存excel
    writer.save()

if __name__ == '__main__':
    filename = '开工报告1'
    file = f"test/{filename}.pdf"
    outfile = f"out/{filename}.xlsx"
    extract_pdf_table(input_file=file, output_file=outfile)


# -*- coding: utf-8 -*-
'''
Created on 2020-11-16

@author: zhys513(254851907@qq.com)
''' 
from opspipe.app.core.logging.logging import LoggerFactory
logger = LoggerFactory('test').logger
  
logger.info(1111)


# -*- coding: utf-8 -*-
'''
Created on 2020-11-9

@author: zhys513(254851907@qq.com)
'''

from opspipe.reg.scanner import regularExtrator
from opspipe.reg.common import regularExtrator as cRegularExtrator
from opspipe.base.logger import logtime 
'''
资质证书名称、企业名称、详细地址、统一社会信用代码、法定代表人、注册资本、经济性质、证书编号、有效期
'''

# 资质证书名称
def get_qualification_certificate(txt): 
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('企业名称:')
        #ex.print_list()
        ex.text = ex.data_list[0]
        ex.clear_txt(r'\n')
    except:
        ex.text = ''
    return ex.text


# 企业名称
def get_enterprise_name(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('企业名称:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt('\n')  
        ex.text = ex.data_list[0]
    except:
        ex.text = ''
    return ex.text


# 详细地址
def get_addr_detailed(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('详细地址:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt('\n')  
        ex.text = ex.data_list[0]
    except:
        ex.text = ''
    return ex.text

 
# 统一社会信用代码
def get_unified_social_credit_code(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('法定代表人:')
        ex.text = ex.data_list[0]
        ex.split_txt('\n')  
        #ex.print_list() 
        ex.text = ex.scan_for_lists_p('\d{6}')[0]
        cre = cRegularExtrator(ex.text)
        ex.text = cre.remove_chinese()
        ex.clear_txt() 
    except:
        ex.text = ''
    return ex.text


# 法定代表人
def get_legal_representative(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('法定代表人:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt('\n')  
        ex.text = ex.data_list[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text


# 注册资本
def get_registered_capital(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('注册资本:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt('\n')  
        ex.text = ex.data_list[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text
 
# 经济性质
def get_economic_nature(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('经济性质:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt(r'证书编号:')  
        ex.text = ex.data_list[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text

#证书编号
def get_certificate_number(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('证书编号:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt(r'有效期:')  
        ex.text = ex.data_list[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text

#有效期
def get_validity_date(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('有效期:')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt(r'发证机关:')  
        ex.text = ex.data_list[0]
        ex.text = ex.split_txt('\n')[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text


#  发证机关
def get_post_company(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('发证机关:')  
        ex.text = ex.data_list[1]
        ex.split_txt('\n')  
        ex.data_list = [x for x in ex.data_list if x != '']
        ex.text = ex.data_list[1]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text
# 
# # 发证日期
def get_post_date(txt):
    try:
        ex = regularExtrator(txt) 
        ex.split_txt('发证机关')
        #ex.print_list()
        ex.text = ex.data_list[1]
        ex.split_txt(r'有效期:')  
        ex.text = ex.data_list[0]
        cre = cRegularExtrator(ex.text)
        ex.text = cre.extract_date()[0]
        ex.clear_txt()
    except:
        ex.text = ''
    return ex.text 

@logtime
def get_result(txt):
    
    # 资质证书名称
    qualification_certificate = get_qualification_certificate(txt)

    # 企业名称
    enterprise_name = get_enterprise_name(txt)

    # 详细地址
    addr_detailed = get_addr_detailed(txt)
 
    # 统一社会信用代码
    unified_social_credit_code = get_unified_social_credit_code(txt)

    # 法定代表人
    legal_representative = get_legal_representative(txt)

    # 注册资本
    registered_capital = get_registered_capital(txt)
 
    # 经济性质
    economic_nature = get_economic_nature(txt)

    #证书编号
    certificate_number = get_certificate_number(txt)

    #有效期
    validity_date = get_validity_date(txt)
     
    post_company = get_post_company(txt)
    
    post_date = get_post_date(txt)
    
    result = {"qualification_certificate": qualification_certificate,
              "enterprise_name": enterprise_name,
              "addr_detailed": addr_detailed,
              "unified_social_credit_code": unified_social_credit_code,
              "legal_representative": legal_representative,
              "registered_capital": registered_capital,
              "economic_nature": economic_nature,
              "certificate_number": certificate_number, 
              "validity_date": validity_date,
              "post_company": post_company,
              "validity_date": post_date,
              }
 
    return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-  
import os, json ,uuid ,fitz, requests 
from fastapi import UploadFile, File,Query
from loguru import logger   
from opspipe.app.core.response.BaseResponse import success, fail 
from opspipe.app.main import get_application, get_para
from opspipe.app.settings.config import DEBUG
from opspipe.base.logger import logtime
from opspipe.utils import utils
from tqdm import tqdm
from pydantic import BaseModel
import uvicorn  

def get_result(text):
    None
    
app = get_application()
app.description =  '建设项目基本情况\n项目批复情况\n项目核准批复文件'  
# -------------------- Business --------------------    
 
'''
# 将PDF转化为图片
pdfPath pdf文件的路径
imgPath 图像要保存的文件夹
zoom_x x方向的缩放系数
zoom_y y方向的缩放系数
rotation_angle 旋转角度
'''
@logtime
def pdf_image(pdfPath,imgPath,zoom_x = 5,zoom_y = 5,rotation_angle = 0):
    # 打开PDF文件
    if not os.path.exists(imgPath):
        os.mkdir(imgPath) 
        
    pdf = fitz.open(pdfPath)
    D = []
    # 逐页读取PDF
    for pg in tqdm(range(0, pdf.pageCount)):
        page = pdf[pg]
        # 设置缩放和旋转系数
        trans = fitz.Matrix(zoom_x, zoom_y).preRotate(rotation_angle)
        pm = page.getPixmap(matrix=trans, alpha=False)
        # 开始写图像
        f = os.path.join(imgPath,str(pg)+".png") 
        pm.writePNG(f) 
        D.append(f)
    pdf.close()
    
    return D


@logtime
def ocr(path):
    url = "http://192.168.55.199:30007/icr/recognize_document" 
    files = {'filename': open(path, 'rb')}
    r = requests.post(url, files=files) 
    dt = json.loads(r.text) 
    linesText = dt['linesText']
    return '\n'.join(linesText) ,r.text
 
class Item(BaseModel):
    text: str = Query(
        '',
        description="待抽取文本",
        deprecated=True
    )
 
@app.post("/predict",summary="纯文本抽取接口")
async def predict(item: Item):
    text = item.dict()['text']
    result = get_result(text)
    return success(result)

@app.post("/predict_img", summary="图片抽取文本接口",)
async def predict_img(file: UploadFile = File(...)):  
    img_path = utils.save_file(file)
    try:     
        txt,json = ocr(img_path) 
        rs = {'txt' : txt,
              'result' : json
            }
        logger.info(txt) 
        return success(rs)
    
    except Exception as err: 
        logger.error(f"{err}")
        # TOTO 优化500 全局错误 
        return fail(400)
    
    
@app.post("/predict_pdf", summary="pdf扫描件抽取文本接口") 
async def predict_pdf(file: UploadFile = File(...)):  
    pdf_path = utils.save_file(file)
    try:  
        uid = './' + str(uuid.uuid4()).replace('-','')
        D = pdf_image(pdf_path,uid) 
        txts = []
        rss = []
        for i,img_path in enumerate(D): 
            txt,json = ocr(img_path)
            txts.append(txt) 
            rss.append(json)
            logger.info(str(i) + '/'+ str(len(D)) + '>>>'+ txt)
        rs = {'txt' : ''.join(txts),
              'results' : rss
            }
        return success(rs)
    
    except Exception as err: 
        logger.error(f"{err}")
        # TOTO 优化500 全局错误
        return fail(400)
     
   
if __name__ == '__main__': 
    host , port = get_para()
    uvicorn.run(app='__main__:app', host=host, port=port, reload=True, debug=DEBUG)



# -*- coding: utf-8 -*-
import re 
from .spliter import RegularExtrator as pre

class RegularExtrator(pre):

    def __init__(self, text='',data_list = []):
        pre.__init__(self, text, data_list)  
        self.text = text
        self.data_list = data_list 
    
    '''
      从list中查询包含字符列表的项  多种组合全部满足才匹配成功
    lists:  ['',''],['','']
    '''
    def scan_for_lists(self,*lists:list):
        data_list = self.flatten(self.data_list)
        data_list_out = []
        for item in data_list: 
            mt = True
            for list_p in lists:
                if not any(word in item for word in list_p): 
                    mt = False 
            if mt :
                data_list_out.append(item)
        self.data_list = data_list_out
        return self.data_list 

    '''
      从list中匹配正则列表的项 ,多项正则全部匹配才正确
    '''
    def scan_for_lists_p(self,*patterns:str): 
        data_list = self.flatten(self.data_list)
        data_list_out = []
        for item in data_list: 
            mt = True
            for pattern in patterns:
                match = re.search(pattern, item)
                if not match: 
                    mt = False
            if mt:
                data_list_out.append(item)
        self.data_list = data_list_out
        return self.data_list
    
    '''
    从文本抽取单个
    '''
    def match_txt_o(self,pattern:str): 
        match = re.findall(pattern, self.text)
        if match:
            self.text = match[0]
        else:
            self.text = ''
        return self.text
    
    
    '''
    从文本抽取多个
    '''
    def match_txt_m(self,pattern:str):
        self.data_list = re.findall(pattern, self.text)    
        return self.data_list
    
    '''
    从list抽取单个
    '''
    def match_list_o(self,pattern:str): 
        data_list = self.flatten(self.data_list) 
        for items in data_list:
            match = re.search(pattern, items)
            if match: 
                self.text = match.group()   
            else:
                self.text = '' 
        return self.text
                
    
    '''
    从list抽取多个
    '''
    def match_list_m(self,pattern:str): 
        data_list = self.flatten(self.data_list)
        data_list_out = []
        for item in data_list: 
            match = re.findall(pattern, item)  
            data_list_out = data_list_out + match
        self.data_list = data_list_out
        return self.data_list


# encoding: utf-8
"""
@author: linvaux
@contact: linvaux@outlook.com
@time: 2020/2/22 22:03
@desc: 日志
"""

import datetime
import logging
import os
from logging import handlers

from ...settings.config import LOG_DIR

try:
    log_level = os.environ['LOG_LEVEL']
except KeyError:
    log_level = 'info'  # pylint: disable=invalid-name (C0103)


class LoggerFactory:
    """
    Manage log
    """
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warn': logging.WARNING,
        'error': logging.ERROR,
    }

    def __init__(self, filename='log', when='D', back_count=3,
                 fmt='[%(levelname)s] - %(asctime)s - process: %(process)d - %(name)s：%(lineno)d - %(message)s'):
        """
        保存日志
        Args:
            filename: log file name
            when: saved by day
            back_count:
            fmt: log format
        """
        if not os.path.exists(LOG_DIR):
            os.mkdir(LOG_DIR)
            #  这里进行判断，如果logger.handlers列表为空，则添加，否则，直接去写日志
        today = datetime.datetime.now().strftime("%Y-%m-%d")
        log_file = os.path.join(LOG_DIR, filename + "-" + today + ".log")
        self.logger = logging.getLogger(filename) 
        if not self.logger.handlers:
            format_str = logging.Formatter(fmt)
            self.logger.setLevel(self.level_relations[log_level])
            console_handler = logging.StreamHandler()
            console_handler.setFormatter(format_str)
            file_handler = handlers.TimedRotatingFileHandler(filename=log_file, when=when, backupCount=back_count,
                                                             encoding='utf-8')
            file_handler.setFormatter(format_str)
            self.logger.addHandler(console_handler)
            self.logger.addHandler(file_handler)
         



# -*- coding: utf-8 -*-
import re 
from .cleaner import RegularExtrator as cre

class RegularExtrator(cre):

    def __init__(self, text='',data_list = []):
        cre.__init__(self, text, data_list)  
        self.text = text
        self.data_list = data_list 
    
    def split_list(self,pattern_split= r'[，。；]'): 
        data_list = self.flatten(self.data_list)
        data_list_out = []
        for item in data_list: 
            txt_split = re.split(pattern_split, item) 
            data_list_out.append(txt_split)
        self.data_list = data_list_out
        return self.data_list
    
    def split_txt(self,pattern_split= r'[，。；]',retain = False): 
        sentences = re.split(pattern_split, self.text) 
        sentences.append("")
        if retain :
            sentences = ["".join(i) for i in zip(sentences[0::2],sentences[1::2])]
        self.data_list = sentences
        return self.data_list
    
    def seg_generator(self,seg_len, seg_backoff=0):   # 滑窗算法
        data_list_out = []
        iterables = (self.text, )
        if seg_len <= 0:
            #yield iterables, 0
            data_list_out = [self.text]
        else:
            #  # 确保iterables列表中每一项的条目数相同
            #  assert sum([len(x)
            #              for x in iterables]) == len(iterables[0]) * len(iterables)
            assert iterables[0] is not None
            s0 = 0
            while s0 < len(iterables[0]):
                s1 = s0 + seg_len
                segs = [x[s0:s1] if x else None for x in iterables]
                #yield segs, s0
                s0 += seg_len - seg_backoff 
                data_list_out.append(segs[0])
                
        self.data_list = data_list_out
        
        return self.data_list


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
 nohup python run_task5.py --do_experiment --data_dir data > logs/log.log 2>&1 &
'''
import os, sys, json, random, copy
# os.environ["CUDA_VISIBLE_DEVICES"] = "1"
import os, sys
from collections import Counter
import pandas as pd
from tqdm import tqdm
from loguru import logger
from theta.utils import load_json_file, split_train_eval_examples
from theta.modeling import LabeledText, load_ner_examples, load_ner_labeled_examples, save_ner_preds, show_ner_datainfo
from pathlib import Path
import utils_theta_submission
from ...utils import read_txts

#  if os.environ['NER_TYPE'] == 'span':
#      from theta.modeling.ner_span import load_model, get_args
#  else:
#      from theta.modeling.ner import load_model, get_args

dataset_dir = 'datasets'
dataset_name = 'ccf2020_business_ner'
#read_txts(Path(dataset_dir,dataset_name,'labels.txt'))
ner_labels =[
    'game',
    'scene',
    'vx',
    'name',
    'company',
    'organization',
    'mobile',
    'movie',
    'email',
    'QQ',
    'government',
    'position',
    'book',
    'address'
    ]

# -------------------- Data --------------------

def clean_text(text):
    text = str(text).replace('\n',' ')
    # if text:
    #    text = text.strip()
    # text = re.sub('\n', ' ', text)
    return text

def train_data_generator(train_file):
    lines = load_json_file(train_file)

    for i, x in enumerate(tqdm(lines)):
        guid = str(i)
        text = clean_text(x['originalText'])
        sl = LabeledText(guid, text)

        entities = x['entities']
        for entity in entities:
            start_pos = entity['start_pos']
            end_pos = entity['end_pos'] - 1
            category = entity['label_type']
            mention = entity['mention']

            # 矫正前后带空格
            ent = text[start_pos:end_pos + 1]
            if mention != ent:
                print('###', mention, ent)
            if len(ent.lstrip()) != len(ent):
                l = len(ent) - len(ent.lstrip())
                start_pos = start_pos + l
                print(ent, '--lstrip--', text[start_pos:end_pos + 1])

            if len(ent.rstrip()) != len(ent):
                l = len(ent) - len(ent.rstrip())
                end_pos = end_pos - l
                print(ent, '--rstrip--', text[start_pos:end_pos + 1])

            if category not in ner_labels:
                continue

            sl.add_entity(category, start_pos, end_pos)

        yield str(i), text, None, sl.entities


'''
def load_train_val_examples(args):

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})



    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples 
'''


def load_train_val_examples(args):
    lines = []
    for guid, text, _, entities in train_data_generator(args.train_file):
        sl = LabeledText(guid, text, entities)
        lines.append({'guid': guid, 'text': text, 'entities': entities})

    allow_overlap = args.allow_overlap
    if args.num_augements > 0:
        allow_overlap = False

    train_base_examples = load_ner_labeled_examples(
        lines,
        ner_labels,
        seg_len=args.seg_len,
        seg_backoff=args.seg_backoff,
        num_augements=args.num_augements,
        allow_overlap=allow_overlap)

    train_examples, val_examples = train_base_examples, []
    # 如果训练集和验证集一样就自动切分
    if args.train_file == args.eval_file:
        train_examples, val_examples = split_train_eval_examples(
            train_base_examples,
            train_rate=args.train_rate,
            fold=args.fold,
            shuffle=False)
    else:
        lines = []
        for guid, text, _, entities in train_data_generator(args.eval_file):
            sl = LabeledText(guid, text, entities)
            lines.append({'guid': guid, 'text': text, 'entities': entities})

        val_examples = load_ner_labeled_examples(
            lines,
            ner_labels,
            seg_len=args.seg_len,
            seg_backoff=args.seg_backoff,
            num_augements=args.num_augements,
            allow_overlap=allow_overlap)

    logger.info(f"Loaded {len(train_examples)} train examples, "
                f"{len(val_examples)} val examples.")
    return train_examples, val_examples


def test_data_generator(test_file):
    lines = load_json_file(test_file)
    for i, s in enumerate(tqdm(lines)):
        i = clean_text(s['uid'])
        guid = str(i)
        text_a = clean_text(s['originalText'])

        yield guid, text_a, None, None


def load_test_examples(args):
    test_base_examples = load_ner_examples(test_data_generator,
                                           args.test_file,
                                           seg_len=args.seg_len,
                                           seg_backoff=args.seg_backoff)

    logger.info(f"Loaded {len(test_base_examples)} test examples.")
    return test_base_examples

def generate_submission(args):
    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
    reviews = json.load(open(reviews_file, 'r'))

    submission_file = f"{args.submissions_dir}/{args.dataset_name}_submission_{args.local_id}.json.txt"
    with open(submission_file, 'w') as wt:
        for guid, json_data in reviews.items():
            output_data = {'originalText': json_data['text'], 'entities': []}
            for json_entity in json_data['entities']:
                output_data['entities'].append({
                    'label_type':
                        json_entity['category'],
                    'overlap':
                        0,
                    'start_pos':
                        json_entity['start'],
                    'end_pos':
                        json_entity['end'] + 1,
                    'mention':
                        json_entity['mention']
                })
            output_data['entities'] = sorted(output_data['entities'],
                                             key=lambda x: x['start_pos'])
            output_string = json.dumps(output_data, ensure_ascii=False)
            wt.write(f"{output_string}\n")

    logger.info(f"Saved {len(reviews)} lines in {submission_file}")

    from theta.modeling import archive_local_model
    archive_local_model(args, submission_file)

def to_poplar(args):
    start_pages = 10
    max_pages = 10
    poplar_json = {
        "content": "",
        "labelCategories": [],
        "labels": [],
        "connectionCategories": [],
        "connections": []
    }

    poplar_colorset = [
        '#007bff', '#17a2b8', '#28a745', '#fd7e14', '#e83e8c', '#dc3545',
        '#20c997', '#ffc107', '#007bff'
    ]
    label2id = {x: i for i, x in enumerate(ner_labels)}
    label_categories = poplar_json['labelCategories']
    for _id, x in enumerate(ner_labels):
        label_categories.append({
            "id": _id,
            "text": x,
            "color": poplar_colorset[label2id[x]],
            "borderColor": "#cccccc"
        })

    poplar_labels = poplar_json['labels']
    poplar_content = ""
    eid = 0
    num_pages = 0
    page_offset = 0
    for guid, text, _, entities in train_data_generator(args.train_file):
        if num_pages < start_pages:
            num_pages += 1
            continue

        page_head = f"\n-------------------- {guid} Begin --------------------\n\n"
        page_tail = f"\n-------------------- {guid} End --------------------\n\n"
        poplar_content += page_head + f"{text}" + page_tail

        for entity in entities:
            poplar_labels.append({
                "id":
                    eid,
                "categoryId":
                    label2id[entity.category],
                "startIndex":
                    page_offset + len(page_head) + entity.start,
                "endIndex":
                    page_offset + len(page_head) + entity.end + 1,
            })
            eid += 1

        num_pages += 1
        if num_pages - start_pages >= max_pages:
            break

        page_offset = len(poplar_content)

    poplar_json["content"] = poplar_content
    poplar_json['labels'] = poplar_labels

    poplar_data_file = "./data/poplar_data.json"
    json.dump(poplar_json,
              open(poplar_data_file, 'w'),
              ensure_ascii=False,
              indent=2)
    logger.info(f"Saved {poplar_data_file}")


from theta.modeling import Params, CommonParams, NerParams, NerAppParams, log_global_params

experiment_params = NerAppParams(
    CommonParams(
        dataset_name=dataset_name,
        experiment_name="NER",
        # train_file='E:/dataset/ccks2020_2_task1_train/task1_train.txt',
        # eval_file='E:/dataset/ccks2020_2_task1_train/task1_train.txt',
        # test_file='E:/dataset/ccks2_task1_val/task1_no_val_utf8.txt',
        train_file=Path(dataset_dir, dataset_name, 'train_data.txt'),
        eval_file=Path(dataset_dir, dataset_name, 'train_data.txt'),
        # eval_file='data/task5_dev.txt',
        test_file=Path(dataset_dir, dataset_name, 'test_data.txt'),
        learning_rate=1e-5,
        train_max_seq_length=188,
        eval_max_seq_length=188,
        per_gpu_train_batch_size=32,
        per_gpu_eval_batch_size=32,
        per_gpu_predict_batch_size=32,
        seg_len=186,
        seg_backoff=88,
        num_train_epochs=15,
        fold=9,
        num_augements=2,
        enable_kd=True,
        enable_sda=False,
        sda_teachers=0,
        loss_type="CrossEntropyLoss",  # "FocalLoss",
        model_type="bert",
        model_path=
        "/opt/share/pretrained/pytorch/roberta-wwm-large-ext-chinese",
        # "/opt/share/pretrained/pytorch/bert-base-chinese",
        #  '../../prev_trained_model/bert-wobert-chinese',
        fp16=True,
        best_index="f1",
        seed=6636,
        train_rate=0.9,
        random_type=None),
    NerParams(ner_labels=ner_labels, ner_type='span', no_crf_loss=False))

experiment_params.debug()


def main(args):
    def do_eda(args):
        show_ner_datainfo(ner_labels, train_data_generator, args.train_file,
                          test_data_generator, args.test_file)

    def do_submit(args):
        generate_submission(args)
        ''' 
        from pipeline.ner.utils import theta_submission2poplar
        if not args.data_dir:
            args.data_dir = './' + dataset_dir
        theta_submission2poplar(args)
        '''
        utils_theta_submission.generate_submission_val(args)

    if args.do_eda:
        do_eda(args)

    elif args.do_submit:
        do_submit(args)

    elif args.to_poplar:
        to_poplar(args)

    else:

        # -------------------- Model --------------------
        #  if args.ner_type == 'span':
        #      from theta.modeling.ner_span import NerTrainer
        #  else:
        #      from theta.modeling.ner import NerTrainer

        class AppTrainer(NerTrainer):
            def __init__(self, args, ner_labels):
                super(AppTrainer, self).__init__(args,
                                                 ner_labels,
                                                 build_model=None)

            #  def on_predict_end(self, args, test_dataset):
            #      super(Trainer, self).on_predict_end(args, test_dataset)

        trainer = AppTrainer(args, ner_labels)

        def do_train(args):
            train_examples, val_examples = load_train_val_examples(args)
            trainer.train(args, train_examples, val_examples)

        def do_eval(args):
            args.model_path = args.best_model_path
            _, eval_examples = load_train_val_examples(args)
            model = load_model(args)
            return trainer.evaluate(args, model, eval_examples)

        def do_predict(args):
            args.model_path = args.best_model_path
            test_examples = load_test_examples(args)
            model = load_model(args)
            trainer.predict(args, model, test_examples)
            reviews_file, category_mentions_file = save_ner_preds(
                args, trainer.pred_results, test_examples)
            return reviews_file, category_mentions_file

        if args.do_train:
            do_train(args)

        elif args.do_eval:
            do_eval(args)

        elif args.do_predict:
            do_predict(args)

        elif args.do_experiment:
            '''
            if args.tracking_uri:
                mlflow.set_tracking_uri(args.tracking_uri)
            mlflow.set_exdo_experimentperiment(args.experiment_name)

            with mlflow.start_run(run_name=f"{args.local_id}") as mlrun:
                log_global_params(args, experiment_params)
            '''
            if True:
                # ----- Train -----
                do_train(args)

                # ----- Predict -----
                do_predict(args)

                # ----- Submit -----
                do_submit(args)


if __name__ == '__main__':

    def add_special_args(parser):
        parser.add_argument("--to_poplar", action="store_true")
        return parser

    if experiment_params.ner_params.ner_type == 'span':
        from theta.modeling.ner_span import load_model, get_args, NerTrainer
    else:
        from theta.modeling.ner import load_model, get_args, NerTrainer

    args = get_args(experiment_params=experiment_params,
                    special_args=[add_special_args])
    logger.info(f"args: {args}")

    args.do_eval = True
    main(args)




# -*- coding: utf-8 -*-
'''
Created on 2020-11-6

@author: zhys513(254851907@qq.com)
'''   
import sys,os 
from fastapi import FastAPI, HTTPException
from fastapi.exceptions import RequestValidationError
from .settings import config 
from .core.response.BaseResponse import UnicornException
from .core.response.http_error import http_error_handler, unicorn_exception_handler
from .core.response.validation_error import http422_error_handler 
from .core.events.events import create_stop_app_handler, create_start_app_handler 
from .core.logging.logging import LoggerFactory 
from .core.router.router import router 
from .core.middleware.middleware import init_middlewares
from fastapi.staticfiles import StaticFiles
from fastapi.openapi.docs import (
    get_redoc_html,
    get_swagger_ui_html,
    get_swagger_ui_oauth2_redirect_html,
)
import json,yaml    
from .core.openapi.converter import Converter   

# 初始化应用
application = FastAPI(title=config.PROJECT_NAME, version=config.VERSION, description=config.DESCRIPTION,docs_url=None, redoc_url=None)

@application.get("/docs", include_in_schema=False)
async def custom_swagger_ui_html():
    return get_swagger_ui_html(
        openapi_url=application.openapi_url,
        title=application.title + " - Swagger UI",
        oauth2_redirect_url=application.swagger_ui_oauth2_redirect_url,
        swagger_js_url=f"/{config.SWAGGER}/swagger-ui-bundle.js",
        swagger_css_url=f"/{config.SWAGGER}/swagger-ui.css",
    )


@application.get(application.swagger_ui_oauth2_redirect_url, include_in_schema=False)
async def swagger_ui_redirect():
    return get_swagger_ui_oauth2_redirect_html()


@application.get("/redoc", include_in_schema=False)
async def redoc_html():
    return get_redoc_html(
        openapi_url=application.openapi_url,
        title=application.title + " - ReDoc",
        redoc_js_url=f"/{config.SWAGGER}/redoc.standalone.js",
    ) 
 
# 注入
def get_application() -> FastAPI:
    # Log
    application.state.log = LoggerFactory('log').logger
    # 添加事件
    application.add_event_handler("startup", create_start_app_handler(application))
    application.add_event_handler("shutdown", create_stop_app_handler(application))
    # 中间件
    init_middlewares(application)
    # 路由
    application.include_router(router)
    # 添加错误处理
    application.add_exception_handler(HTTPException, http_error_handler)
    application.add_exception_handler(RequestValidationError, http422_error_handler) 
    application.add_exception_handler(UnicornException, unicorn_exception_handler) 
    # swagger 本地化
    swagger_dir = os.path.join(os.path.abspath(os.path.dirname(__file__)),config.SWAGGER)
    application.mount(f"/{config.SWAGGER}", StaticFiles(directory=swagger_dir), name=f"/{config.SWAGGER}")

    return application
    
def create_md_doc():  
    if not os.path.exists(config.STATIC_DIR):
        os.mkdir(config.STATIC_DIR)
        
    openapi = application.openapi()
    openapi_str = json.dumps(openapi, ensure_ascii=False)
    dyaml = yaml.load(openapi_str) 
    dyaml = yaml.dump(dyaml,default_flow_style=False)
    converter = Converter()
    text = converter.convert_content(dyaml)    
    with open(config.MDDOC_DIR, 'w', encoding='utf-8') as out:
        out.write(text)

def get_para(host = "0.0.0.0",port = 5000):
    create_md_doc()    
    if len(sys.argv) != 1:
        for argv in sys.argv[1:]:
            if argv[:6] == "--port":
                port = int(argv.rsplit("=")[1])
            elif argv[:6] == "--host":
                host = argv.rsplit("=")[1]
    return host,port


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .poplar import theta_train2poplar,theta_submission2poplar
from .poplar import theta2poplar
from .theta import poplar2theta
from .bio import theta2bio


'''
转换比赛数据
json - labelImg标注xml
为imgaug数据扩展做准备
'''
import json,os,math

def mkdir(path):
    # 去除首位空格
    path = path.strip()
    # 去除尾部 \ 符号
    path = path.rstrip("\\")
    # 判断路径是否存在
    # 存在     True
    # 不存在   False
    isExists = os.path.exists(path)
    # 判断结果
    if not isExists:
        # 如果不存在则创建目录
        # 创建目录操作函数
        os.makedirs(path)
        print(path , ' 创建成功')
        return True
    else:
        # 如果目录存在则不创建，并提示目录已存在
        print(path , ' 目录已存在')
        return False

# json.load()函数的使用，将读取json信息
file = open('annotations/tablebank_latex_test.json','r',encoding='utf-8')
data = json.load(file)
categories = {item.get('id', 'NA'):item.get('name', 'NA') for item in data['categories']}
#print(categories[1])

images = {item.get('id', 'NA'):{'id':item.get('id', 'NA'),'file_name':item.get('file_name', 'NA'),'height':item.get('height', 'NA'),'width':item.get('width', 'NA'),'license':item.get('license', 'NA')} for item in data['images']}
#images = [(item.get('id', 'NA'),item.get('file_name', 'NA'),item.get('height', 'NA'),item.get('width', 'NA'),item.get('license', 'NA')) for item in data['images']]

#annotations = {item.get('id', 'NA'):{'id':item.get('id', 'NA'),'image_id':item.get('image_id', 'NA'),'category_id':item.get('category_id', 'NA'),'iscrowd':item.get('iscrowd', 'NA'),'segmentation':item.get('segmentation', 'NA'),'area':item.get('area', 'NA'),'bbox':item.get('bbox', 'NA')} for item in data['annotations']}

#annotations = [(item.get('id', 'NA'),item.get('image_id', 'NA'),item.get('category_id', 'NA'),item.get('iscrowd', 'NA'),item.get('segmentation', 'NA'),item.get('area', 'NA'),item.get('bbox', 'NA')) for item in data['annotations']]
#print(annotation[11])
xml_path = 'Pascal_VOC/Annotations/' #注意后面有/
mkdir(xml_path)
imgs_dir = 'Pascal_VOC/JPEGImages/' #注意后面有/
mkdir(imgs_dir)
trains = {}
for item in data['annotations'] :
    image_id = item.get('image_id', 'NA')
    file_name = images[image_id]['file_name']
    category_id = item.get('category_id', 'NA')
    bbox = item.get('bbox', 'NA') 
    if trains.__contains__(image_id): 
        file_bboxs = trains[image_id] + ' ' + ','.join(str(s) for s in bbox if s not in [None,'']) 
        file_bboxs = file_bboxs + ',' +  str(category_id)
        trains[image_id] = file_bboxs 
    else: 
        file_bboxs = file_name + ' ' + ','.join(str(s) for s in bbox if s not in [None,'']) 
        file_bboxs = file_bboxs + ',' +  str(category_id)
        trains[image_id] = file_bboxs

for key in trains:
    file_bboxs = trains[key].split( ) 
    file_name = os.path.splitext(file_bboxs[0])[0]
    num = len(file_bboxs)
    train_txt = open(xml_path + file_name + ".xml",'w+')  
    print('<annotation verified="no">',file = train_txt)
    print("    <folder>Annotations</folder>",file = train_txt)
    print("    <filename>" + file_bboxs[0] + "</filename>",file = train_txt)
    print("    <path>"  + imgs_dir + file_bboxs[0] + "</path>",file=train_txt)
    print("    <source>",file=train_txt)
    print("        <database>Unknown</database>",file=train_txt)
    print("    </source>",file=train_txt)
    print("    <size>",file=train_txt)
    print("        <width>"+str(images[key]['width'])+"</width>",file=train_txt)
    print("        <height>"+str(images[key]['height'])+"</height>",file=train_txt)
    print("        <depth>3</depth>",file=train_txt)
    print("    </size>",file=train_txt)
    print("    <segmented>0</segmented>",file=train_txt)
 
    for i in range(1,num): 
        bbox = file_bboxs[i].split(',') 
        if len(bbox)>3:
            print("    <object>",file=train_txt)
            print("        <name>"+str(bbox[4])+"</name>",file=train_txt)
            print("        <pose>Unspecified</pose>",file=train_txt)
            print("        <truncated>0</truncated>",file=train_txt)
            print("        <difficult>0</difficult>",file=train_txt)
            print("        <bndbox>",file=train_txt)
            print("            <xmin>"+str(math.fabs(int(float(bbox[0]))))+"</xmin>",file=train_txt)
            print("            <ymin>"+str(math.fabs(int(float(bbox[1]))))+"</ymin>",file=train_txt)
            print("            <xmax>"+str(math.fabs(int(float(bbox[0]) + float(bbox[2]))))+"</xmax>",file=train_txt)
            print("            <ymax>"+str(math.fabs(int(float(bbox[1]) + float(bbox[3]))))+"</ymax>",file=train_txt)
            print("        </bndbox>",file=train_txt)
            print("    </object>",file=train_txt) 
    print("</annotation>",file=train_txt)
     
    train_txt.close()



# -*- coding: utf-8 -*-
'''
标准数据格式转换工具
'''
from pathlib import Path
from ...utils import read_json

def get_standard_dict(standard_path,
                      sources_file  = 'sources.json',
                      labels_file ='labels.json',
                      labelCategories = 'labelCategories.json',
                      connections = 'connections.json',
                      connectionCategories = 'connectionCategories.json'):
    '''
    转换标准数据为ID字典
    '''
    sources = read_json(Path(standard_path,sources_file))
    labels = read_json(Path(standard_path,labels_file))
    labelCategories = read_json(Path(standard_path,labelCategories))
    connections = read_json(Path(standard_path,connections))
    connectionCategories = read_json(Path(standard_path,connectionCategories))
    labelCategories_dict = {row['id']:row['text'] for row in labelCategories}
    labels_dict = {str(row['id']*1000) + row['srcId']:{'srcId':row['srcId'],'name':row['name'],'categoryId':row['categoryId'],'startIndex':row['startIndex'],'endIndex':row['endIndex']} for row in labels}
    sources_dict = {row['id']:{'title':row['title'],'content':row['content']} for row in sources}
    connectionCategories_dict = {row['id']:row['text'] for row in connectionCategories}
    return connections,labelCategories_dict,labels_dict,sources_dict,connectionCategories_dict


# -*- coding: utf-8 -*-
'''
Created on 2020-9-24
先到标注工具新建类型，导入json，再导出json作为model.json

@author: zhys513(254851907@qq.com)
'''
import json,os
from pathlib import Path
from ...utils import read_json

def poplar2theta(poplar_path, output_path, train_file='train_data.txt', labels_file='labels.txt'):
    '''标注数据 转 theta
    可配置'''
    if not os.path.exists(output_path):
        os.mkdir(output_path)

    labels, output_datas = [], []
    poplar_path_name = [[Path(poplar_path, i), i.split('.')[0]] for i in os.listdir(poplar_path) if
                        i.split('.')[-1] == 'json']
    for p, i in poplar_path_name:
        poplar = read_json(p)
        guid = poplar['id']
        originalText = poplar['content']
        title = poplar['title']
        labelCategories_dict = {row['id']: row['text'] for row in poplar['labelCategories']}
        poplars = poplar['labels']
        output_data = {'guid': guid, 'title': title, 'originalText': originalText, 'entities': []}
        for epos in poplars:
            # eid = epos['id']
            categoryId = epos['categoryId']
            start_pos = epos['startIndex']
            end_pos = epos['endIndex']
            label_type = labelCategories_dict[categoryId]
            output_data['entities'].append({
                'label_type': label_type,
                'overlap': 0,
                'start_pos': start_pos,
                'end_pos': end_pos,
                'mention': originalText[start_pos:end_pos]
            })

            labels.append(label_type)

        output_datas.append(output_data)

        with open(Path(output_path, train_file), 'w', encoding='utf-8') as wt:
            for output_data in output_datas:
                output_string = json.dumps(output_data, ensure_ascii=False)
                wt.write(f"{output_string}\n")

    with open(Path(output_path, labels_file), 'w', encoding='utf-8') as wt:
        labels = list(set(labels))
        for output_string in labels:
            wt.write(f"{output_string}\n") 



# -*- coding: utf-8 -*-
'''
Created on 2020-12-6
使用 ssh 远程对Docker命令操作
@author: zhys513(254851907@qq.com)
'''

class DockerClient(object): 
    def __init__(self,SSHClient): 
        self.ssh = SSHClient
        
    def build(self,img,memu):
        v_cd = 'cd ' + memu
        v_cmd = 'docker build -t ' + img + ' .'
        if memu:
            v_cmd = v_cd + ' && ' + v_cmd
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
    
    def pull(self,img,usr='',pwd=''):
        v_cmds = 'docker pull ' + img
        if usr:
            v_cmds = ['docker login -u ' + usr + ' -p ' + pwd] + v_cmds
        result = self.ssh.sshExecByMany(v_cmds) 
        return result
        
    def push(self,img,usr='',pwd =''): 
        v_cmds = ['docker push ' + img] 
        if usr:
            v_cmds = ['docker login -u ' + usr + ' -p ' + pwd] + v_cmds
        result = self.ssh.sshExecByMany(v_cmds) 
        return result
     
    def run_cpu(self,img,v_para=''):
        v_cmd = 'docker run -d ' + img + ' ' + v_para
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
    
    def run_gpu(self,img,v_para=''):
        v_cmd = 'nvidia-docker run -d ' + img + ' ' + v_para
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
        
    def start(self,container):
        v_cmd = 'docker start ' + container
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
         
    def ps(self,v_para='-a'):
        v_cmd = 'docker ps ' + v_para 
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
         
    def stop(self,container):
        v_cmd = 'docker stop ' + container
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
    
    def load(self,img):
        v_cmd = 'docker load < ' + img
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
      
    def rm(self,container):
        v_cmd = 'docker rm -f ' + container
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
        
    def rmi(self,img):
        v_cmd = 'docker rmi -f ' + img
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
         
    def logs(self,container,tail = 0):
        v_cmd = 'docker logs ' + container
        if tail > 0: 
            v_cmd = 'docker logs --tail=' + str(tail) + ' ' + container
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
    
    
    def pid(self,container):
        v_cmd = "docker inspect -f '{{.State.Pid}}'" + container 
        result = self.ssh.sshExecByOne(v_cmd) 
        return result
 
        
    
    


# -*- coding: utf-8 -*-
'''
Created on 2020-11-6

@author: zhys513(254851907@qq.com)
'''
import requests,json
''' OCR
url = "http://192.168.55.199:30007/icr/recognize_document"
path = "C:\\Users\\Administrator\\Desktop\\train\\images\\images0.png"
files = {'filename': open(path, 'rb')}
r = requests.post(url, files=files)
print(r.url)
print(r.text)
txt = r.text
dt = json.loads(txt) 

linesText = dt['linesText']
print(linesText)
'''


url = "http://192.168.49.138:8080/converTxt" 
files = {'file': open('C:\\Users\\Administrator\\Desktop\\opspipe-0.0.3\\1.ceb', 'rb')}
r = requests.post(url, files=files) 
dt = json.loads(r.text) 
print(dt)
linesText = dt['content'] 


# -*- coding: utf-8 -*-
'''
Created on 2020-9-29
csv 标注数据转换为 theta ner训练数据
@author: zhys513(254851907@qq.com)
'''
import json 
from pathlib import Path 
import pandas as pd
from collections import Counter
from loguru import logger
 
def generate_submission(args):
    all_ner_labels = []
    reviews_file = f"{args.latest_dir}/{args.dataset_name}_reviews_{args.local_id}.json"
    reviews = json.load(open(reviews_file, 'r', encoding='utf-8'))

    output_datas = []
    submission_file = f"{args.submissions_dir}"

    for guid, json_data in reviews.items():  # T1    DRUG_DOSAGE 111 113    丸剂
        originalText = json_data['text']

        for _, json_entity in enumerate(json_data['entities']):
            # T = 'T' + str(i + 1)
            label_type = json_entity['category']
            start_pos = int(json_entity['start'])
            end_pos = int(json_entity['end'])
            entity = originalText[start_pos:end_pos + 1]

            if 0 < len(entity) and len(entity) < 80:
                output_data = [str(guid), label_type, start_pos, end_pos, '0']
                output_datas.append(output_data)
                all_ner_labels.append(label_type)

    df = pd.DataFrame(output_datas, columns=['﻿ID', 'Category', 'Pos_b', 'Pos_e', 'Privacy'])
    df.to_csv(Path(submission_file, 'predict.csv'), index=None, encoding='UTF-8')

    logger.info(f'Count: {len(output_datas)} {Counter(all_ner_labels)}')

    logger.info(f"Saved {len(reviews)} lines in {submission_file}")

    from theta.modeling import archive_local_model
    archive_local_model(args, submission_file)
 


# -*- coding: utf-8 -*-
from loguru import logger
import time 
import functools
 
# 日志耗时装饰器
def logtime(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = time.perf_counter()
        res = func(*args, **kwargs)
        end = time.perf_counter()
        logger.info('【%s】 took %.2f s' % (func.__name__, (end - start)))
        return res
    return wrapper 

'''
该日志不保存
'''
def lump_logs(guid,text):
    data_begin = f"\n-------------------- {guid} Begin --------------------\n\n"
    data_end = f"\n-------------------- {guid} End --------------------\n\n"
    logger.debug(f"{data_begin} {text} {data_end}")


# -*- coding: utf-8 -*-
'''
Created on 2020-11-11

@author: zhys513(254851907@qq.com)
''' 
#pip install python-docx
from docx.api import Document
import pandas as pd

def extract_docx_table(input_file, output_file):
    """'
    docx的表格提取保存到excel
    :param input_file: 输入的docx文件
    :param output_file: 保存table到excel文件
    :return:
    """
    doc = Document(input_file)
    writer = pd.ExcelWriter(output_file, engine='xlsxwriter')
    # 如果有多个表格，对每个表格都保存成一个sheet
    for i in range(len(doc.tables)):
        table = doc.tables[i]
        data = []
        keys = None
        for j, row in enumerate(table.rows):
            text = (cell.text for cell in row.cells)
            if j == 0:
                keys = tuple(text)
                continue
            row_data = dict(zip(keys, text))
            data.append(row_data)
        df = pd.DataFrame(data)
        #保存到sheet
        df.to_excel(writer, sheet_name='table_{}'.format(i))
    writer.save()

if __name__ == '__main__': 
    
    filename = '石景山区高井规划一路道路工程广宁路15#杆输变电工程勘察设计合同'
    input_file = f"test/{filename}.docx"
    output_file = f"out/{filename}.xlsx"
    extract_docx_table(input_file, output_file)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
from .utils import read_txts,read_txt,read_json
from .utils import load_jsons
from .utils import make_zip,un_zip
from .utils import copy_dir,copy_dirs
from .utils import clean_text
from .base64utils import base64_to_file,base64_to_str


#!/usr/bin/env python
# -*- coding: utf-8 -*- 
from opspipe.ner.theta.core import load_test_one,test_data_generator,load_test_examples
from opspipe.ner.theta.core import trainer_model,do_predict
from opspipe.ner.theta.grid_search import thetaNER


# -*- coding: utf-8 -*-
import json,os
from pathlib import Path
from opspipe.utils import read_json 
from opspipe.pipe.nlp import Base
from tqdm import tqdm

class Theta(Base):
    def __init__(self, data = None): 
        Base.__init__(self, data)  
        #self.data = data
    
    def read(self,path):  
        train_file='train_data.txt'
        labels_file='labels.txt'
         
        
    def converfrom(self,fun):
        self.data = fun.data
    
    def save(self,path):
        assert self.data == None
        print(path)
        
    


# -*- coding: utf-8 -*-
'''
Created on 2020-12-6

@author: zhys513(254851907@qq.com)
'''
from opspipe.ops.deploy.docker_client import DockerClient 
from opspipe.ops.deploy.git_client import GitClient 
from opspipe.ops.deploy.ssh_client import SSHClient 


SSH = SSHClient().sshConnection('192.168.54.22','root', 'yirong123',10022)
DOCKER = DockerClient(SSH)
GIT = GitClient(SSH)
result = GIT.clone("http://192.168.55.41/ai/cv-object-detection-safety-clothes.git")
#SSH.close()
result = SSH.sshExecByMany(['cd cv-object-detection-safety-clothes','pwd'])
 
'''
result = DOCKER.stop('ai_special_1.0')
print(result)
result = DOCKER.start('ai_special_1.0')
print(result)
'''
#result = DOCKER.logs('ai_special_1.0',10)
print(result)


"""基础返回结构"""


def base_response(code, msg, data=None):
    """基础返回格式"""
    if data is None:
        data = {}
    result = {
        "code": code,
        "message": msg,
        "data": data
    }
    return result


def success(data=None, msg='Success'):
    """成功返回格式"""
    return base_response(200, msg, data)


def fail(code=-1, msg='Fail', data=None):
    """失败返回格式"""
    return base_response(code, msg, data)


class UnicornException(Exception):

    def __init__(self, code, errmsg, data=None):
        """
        失败返回格式
        :param code:
        :param errmsg:
        """
        if data is None:
            data = {}
        self.code = code
        self.errmsg = errmsg
        self.data = data



# -*- coding: utf-8 -*-
from tqdm import tqdm
from loguru import logger
from theta.utils import load_json_file  
from theta.modeling.ner_utils import InputExample  
from theta.utils import seg_generator 
from theta.modeling import load_ner_examples,get_ner_preds_reviews 

def do_predict(args,trainer,model,txt):
    test_examples = load_test_one(args,txt)
    trainer.predict(args, model, test_examples)
    reviews, _ = get_ner_preds_reviews(trainer.pred_results, test_examples, args.seg_len, args.seg_backoff)
    return reviews

def trainer_model(experiment_params,ner_labels):
    def add_special_args(parser):
        parser.add_argument("--to_poplar", action="store_true")
        return parser
    
    if experiment_params.ner_params.ner_type == 'span':
        from theta.modeling.ner_span import load_model, get_args, NerTrainer
    else:
        from theta.modeling.ner import load_model, get_args, NerTrainer
    
    args = get_args(experiment_params=experiment_params,
                    special_args=[add_special_args])
    logger.info(f"args: {args}")
    
    class AppTrainer(NerTrainer):
        def __init__(self, args, ner_labels):
            super(AppTrainer, self).__init__(args, ner_labels, build_model=None)
    
    trainer = AppTrainer(args, ner_labels)
    
    #args.model_path = args.best_model_path
    model = load_model(args) 
    
    return trainer,model,args
    
# -------------------- Data -------------------- 
def test_data_generator(test_file):
    lines = load_json_file(test_file)
    for i, s in enumerate(tqdm(lines)): 
        guid = str(i)
        try:
            guid =  s['guid']
        except :
            None
             
        text_a = s['originalText']

        yield guid, text_a, None, None


def load_test_examples(args):
    test_base_examples = load_ner_examples(test_data_generator,
                                           args.test_file,
                                           seg_len=args.seg_len,
                                           seg_backoff=args.seg_backoff)

    logger.info(f"Loaded {len(test_base_examples)} test examples.")
    return test_base_examples 

def load_test_one(args,text_a): 
    examples = []
    for (seg_text_a, ), text_offset in seg_generator((text_a, ), args.seg_len,
                                                     args.seg_backoff):
        examples.append(
            InputExample(guid=str(1),
                         text_a=seg_text_a,
                         labels='',
                         text_offset=text_offset))
    logger.info(f"Loaded {len(examples)} examples.")

    return examples


#读取图片，初步处理，二值化
#获取表格线
#获取轮廓并筛选出单元格的轮廓
#根据单元格轮廓利用掩膜筛选出需要定位的文本
#定位文本并框选出来

#coding = utf-8
import cv2
import numpy as np

class ReadTable:
    kernel_rect_3_3 = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 3))

    def __init__(self):
        pass

    def thres_img(self,img_path,max_threshold = 232,text_threshold = 190,close_num = 1,dnmt_hor=50,dnmt_ver=50,is_noise_point = False):
        """
        [外方]将src单阈值二值化，筛去色块，获得纯粹黑色线条，并做反向操作，获得黑底白线，供后续使用
        :param img_path: 输入的图像地址
        :param max_threshold: 二值化，获取表格线框时的最大阈值
        :param text_threshold: 二值化，获取表格全部内容的时候的阈值
        :param close_num: 开闭操作的次数
        :return:
        """
        assert isinstance(img_path,str) and isinstance(max_threshold,int)
        assert isinstance(text_threshold,int) and isinstance(close_num,int)
        src = cv2.imread(img_path)
        # gray_src = cv2.cvtColor(src,cv2.COLOR_BGR2GRAY)
        gray_src = self.remove_red(src)
        cv2.imwrite(r'..\data\gray_src.png', gray_src)
        _, thresh_src = cv2.threshold(gray_src, max_threshold, 255, cv2.THRESH_BINARY)
        cv2.bitwise_not(thresh_src, thresh_src)  # 取反
        cv2.imwrite(r'..\data\thresh_src1.png', thresh_src)
        thresh_src2 = self.get_lines(thresh_src, dnmt_hor,dnmt_ver)
        cv2.imwrite(r'..\data\thresh_src2.png', thresh_src2)
        clean_src = cv2.morphologyEx(thresh_src2, cv2.MORPH_CLOSE, self.kernel_rect_3_3, None, None, close_num)
        _, thresh_src = cv2.threshold(gray_src, text_threshold, 255, cv2.THRESH_BINARY)
        cv2.bitwise_not(thresh_src, thresh_src)  # 取反
        cv2.imwrite(r'..\data\thresh_src.png',thresh_src)
        if is_noise_point:
            kerner1 = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 1))
            src1 =cv2.morphologyEx(thresh_src,cv2.MORPH_OPEN,kerner1,None,None,1)
            kerner2 = cv2.getStructuringElement(cv2.MORPH_RECT, (1, 3))
            src2 = cv2.morphologyEx(thresh_src, cv2.MORPH_OPEN, kerner2, None, None, 1)
            cv2.bitwise_or(src1,src2,thresh_src)
        return thresh_src,clean_src,src

    def remove_red(self, src):
        """
        移除红色印章，并转为灰度图
        :param src: 原始图片
        :return:
        """
        assert isinstance(src,np.ndarray)
        # 利用hsv方法将红色部分提取出来
        hsv_src = cv2.cvtColor(src, cv2.COLOR_BGR2HSV)
        lower_red = np.array([140, 25, 0])#红色部分的参数区间，包含两部分
        upper_red = np.array([180, 255, 255])
        mask = cv2.inRange(hsv_src, lower_red, upper_red)
        lower_red1 = np.array([0, 25, 0])
        upper_red1 = np.array([10, 255, 255])
        mask1 = cv2.inRange(hsv_src, lower_red1, upper_red1)
        mask_all = mask + mask1
        res = cv2.bitwise_and(src, src, mask=mask_all)
        gray_red = cv2.cvtColor(res, cv2.COLOR_BGR2GRAY)
        cv2.imwrite(r'..\data\gray_red.png', gray_red)

        #将原图的红色部分设置为白色
        gray_src = cv2.cvtColor(src, cv2.COLOR_BGR2GRAY)
        rows, cols = gray_red.shape
        for i in range(rows):
            for j in range(cols):
                value = gray_red[i, j]
                if value != 0:
                    gray_src[i, j] = 255
        cv2.imwrite(r'..\data\gray_src2.png', gray_src)

        #通道分离，仅对红色通道进行阈值处理
        b_channel, g_channel, r_channel = cv2.split(src)
        _, thresh_src1 = cv2.threshold(r_channel, 200, 255, cv2.THRESH_BINARY)
        cv2.imwrite(r'..\data\thresh_src11.png', thresh_src1)

        #将上面得到的阈值处理后的图片增添到灰度图中.
        for i in range(rows):
            for j in range(cols):
                value = thresh_src1[i, j]
                value_gray = gray_src[i,j]
                if value == 0 and value_gray !=0:#当二值化图thresh_src1中为黑色,灰度图中对应位置不为黑色时，将gray_src对应的位置也改成黑色
                    gray_src[i, j] = 0
        cv2.imwrite(r'..\data\gray_src3.png', gray_src)
        return gray_src

    def get_lines(self,thresh_src,dnmt_hor,dnmt_ver):
        """
        [秘方]获取表格线
        :param thresh_src:
        :return:
        """
        assert isinstance(thresh_src,np.ndarray)
        #获取水平线
        shape_hor = int(thresh_src.shape[0] / dnmt_hor)
        if shape_hor == 0:
            shape_hor = 1
        kernel_hor = cv2.getStructuringElement(cv2.MORPH_RECT, (shape_hor, 1))  # 获取n*1的矩形结构元素
        bin_src_hor = cv2.morphologyEx(thresh_src, cv2.MORPH_OPEN, kernel_hor, None, None, 1)  # 做一次开操作，先腐蚀后膨胀，一次
        bin_src_hor = cv2.morphologyEx(bin_src_hor, cv2.MORPH_DILATE, kernel_hor, None, None, 1) #继续膨胀，延伸线条

        #获取垂直线
        shape_ver = int(thresh_src.shape[0] / dnmt_ver)
        if shape_ver == 0:
            shape_ver = 1
        kernel_ver = cv2.getStructuringElement(cv2.MORPH_RECT, (1, shape_ver))  # 获取1*n的矩形结构元素
        bin_src_ver = cv2.morphologyEx(thresh_src, cv2.MORPH_OPEN, kernel_ver, None, None, 1)  # 做一次开操作，先腐蚀后膨胀，一次
        bin_src_ver = cv2.morphologyEx(bin_src_ver, cv2.MORPH_DILATE, kernel_ver, None, None, 1) #继续膨胀，延伸线条

        #水平线垂直线合并
        bin_src_mask = bin_src_hor + bin_src_ver  # 水平线和垂直线或操作，都显示在图像里

        #用正方的kernel消除一些局部坑洞
        bin_src_mask = cv2.morphologyEx(bin_src_mask, cv2.MORPH_CLOSE, self.kernel_rect_3_3, None, None, 1)  # 做一次闭操作，先膨胀后腐蚀，一次
        return bin_src_mask


# -*- coding: utf-8 -*-
'''
Created on 2020-11-9

@author: zhys513(254851907@qq.com)
'''
from opspipe.reg.spliter import RegularExtrator 
from unittest import TestCase 
import unittest

text = '急寻特朗普，男孩，于2018年11月27号11时在陕西省安康市汉滨区走失。丢失发型短发，...如有线索，请迅速与警方联系：系18100065143，132-6156-2938，baizhantang@sina.com.cn 和yangyangfuture at gmail dot com'
    
class TestMain(TestCase):
    
    def test_seg_generator(self): 
        ex = RegularExtrator(text) 
        print(ex.clear_txt())
        print(ex.split_txt(r'[：]'))
        print(ex.split_list())
        print(ex.seg_generator(10,5))
        self.assertTrue(len(ex.data_list)>0)

if __name__ == "__main__": 
    unittest.main()


"""
错误异常
"""
from fastapi import HTTPException, Request
from fastapi.responses import JSONResponse

from .BaseResponse import UnicornException


async def http_error_handler(_: Request, exc: HTTPException):
    if exc.status_code == 4000:
        return JSONResponse({
            "code": 4000,
            "message": exc.detail,
            "data": exc.detail
        })
    if exc.status_code == 4003:
        return JSONResponse({
            "code": 4003,
            "message": exc.detail,
            "data": exc.detail
        })
    return JSONResponse({
        "code": exc.status_code,
        "message": exc.detail,
        "data": exc.detail
    }, status_code=exc.status_code)


async def unicorn_exception_handler(_: Request, exc: UnicornException):
    return JSONResponse({
        "code": exc.code,
        "message": exc.errmsg,
        "data": exc.data,
    })




# -*- coding: utf-8 -*-
'''
Created on 2020-11-9

@author: zhys513(254851907@qq.com)
'''
from opspipe.reg.scanner import RegularExtrator
from unittest import TestCase 
import unittest

text = '急寻特朗普，男孩，于2018年11月27号11时在陕西省安康市汉滨区走失。丢失发型短发，...如有线索，请迅速与警方联系：系18100065143，132-6156-2938，baizhantang@sina.com.cn 和yangyangfuture at gmail dot com'
    
class TestMain(TestCase):
    
    def test_scan_for_lists_p(self): 
        ex = RegularExtrator(text)  
        ex.split_txt('，')
        ex.scan_for_lists_p(r'\d{6}') 
        ex.print_list()  
        self.assertTrue(len(ex.data_list)>=1)

if __name__ == "__main__": 
    unittest.main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#from pipeline.data_generator import train_data_generator,load_train_val_examples,test_data_generator,generate_submission
from ..utils import copy_dir
import os,sys
'''
    #当前文件路径
    print(os.path.realpath(__file__))
    #当前文件所在的目录，即父路径
    print(os.path.split(os.path.realpath(__file__))[0])
    #找到父路径下的其他文件，即同级的其他文件
    print(os.path.join(proDir,"config.ini"))
'''

def create_baseline():
    # 当前文件路径
    print(os.path.realpath(__file__))
    # 当前文件所在的目录，即父路径
    print(os.path.split(os.path.realpath(__file__))[0])
    proDir =os.path.split(os.path.realpath(__file__))[0]
    # 找到父路径下的其他文件，即同级的其他文件
    print(os.path.join(proDir, "model"))

    print(sys.argv[0])

#create_baseline()



# -*- coding: utf-8 -*-
from opspipe.base.registry import  Registry, build_from_cfg

RUNNERS = Registry('runner')


def build_runner(cfg, default_args=None):
    return build_from_cfg(cfg, RUNNERS, default_args=default_args)


# -*- coding: utf-8 -*- 
import time,re
import datetime


def validate_date(text):
    '''验证日期格式

    :param text: 待检索文本

    >>> validate_date('2020-05-20')
    True
    >>> validate_date('2020-05-32')
    False
    '''
    try:
        if text != time.strftime('%Y-%m-%d', time.strptime(text, '%Y-%m-%d')):
            raise ValueError
        return True
    except ValueError:
        return False


def validate_datetime(text):
    '''验证日期+时间格式

    :param text: 待检索文本

    >>> validate_datetime('2020-05-20 13:14:15')
    True
    >>> validate_datetime('2020-05-32 13:14:15')
    False
    '''
    try:
        if text != datetime.datetime.strptime(text, '%Y-%m-%d %H:%M:%S').strftime('%Y-%m-%d %H:%M:%S'):
            raise ValueError
        return True
    except ValueError:
        return False


def validateEmail(email):
    email_reg = r'^[a-zA-Z0-9_.-]+[#@][a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\.[.a-zA-Z0-9]{2,6}'
    #email_reg = r'^[a-zA-Z0-9_.-]+@[a-zA-Z0-9-]+(\.[a-zA-Z0-9-]+)*\.[a-zA-Z0-9]{2,6}'
    ret_1 = re.findall(email_reg, email)
    if ret_1:
        return True
    return False


# -*- coding: utf-8 -*-
class RegularExtrator_txt():

    def __init__(self, text='',data_list = []):
        self.text = text
      
    def select_txt(self,start_index = 0, end_index = 0): 
        # 文本 范围选取
        txt_len = len(self.text)
        if end_index == 0:
            end_index = txt_len 
        self.text = self.text[start_index:end_index]   
        return self.text
    
    def set_txt(self,txt):
        self.text = txt
        return self  
    
    def print_txt(self,uid='-'):
        print(f'text({uid})>>>',self.text)
           

class RegularExtrator(RegularExtrator_txt):

    def __init__(self, text='',data_list = []):
        RegularExtrator_txt.__init__(self, text)
        self.text = text
        self.data_list = data_list
    
    def select_list(self,start_index = 0, end_index = 0): 
        # list 范围选取
        list_len = len(self.data_list)
        if end_index == 0:
            end_index = list_len 
        self.data_list = self.data_list[start_index:end_index]   
        return self.data_list
      
    def set_list(self,data_list):
        self.data_list = data_list
        return self
     
         
    def print_list(self,uid='-'):
        print(f'list({uid})>>>',self.data_list)
        
    def flatten(self,nested):
        try:
            #不要迭代类似字符串的对象：
            try:
                nested+'' 
            except TypeError:
                pass
            else:
                raise TypeError
    
            for sublist in nested:
                for element in self.flatten(sublist):
                    yield element
        except TypeError:
            yield nested


import os
import random


def mkdir(path):
    # 去除首位空格
    path = path.strip()
    # 去除尾部 \ 符号
    path = path.rstrip("\\")
    # 判断路径是否存在
    # 存在     True
    # 不存在   False
    isExists = os.path.exists(path)
    # 判断结果
    if not isExists:
        # 如果不存在则创建目录
        # 创建目录操作函数
        os.makedirs(path)
        print(path + ' 创建成功')
        return True
    else:
        # 如果目录存在则不创建，并提示目录已存在
        print(path + ' 目录已存在')
        return False


trainval_percent = 0.66
train_percent = 0.5
xmlfilepath = 'Annotations'
txtsavepath = 'ImageSets/Main'
mkdir(txtsavepath)
total_xml = os.listdir(xmlfilepath)

num=len(total_xml)
list=range(num)
tv=int(num*trainval_percent)
tr=int(tv*train_percent)
trainval= random.sample(list,tv)
train=random.sample(trainval,tr)

ftrainval = open('ImageSets/Main/trainval.txt', 'w')
ftest = open('ImageSets/Main/test.txt', 'w')
ftrain = open('ImageSets/Main/train.txt', 'w')
fval = open('ImageSets/Main/val.txt', 'w')

for i  in list:
    name = total_xml[i][:-4] + '\n'
    if i in trainval:
        ftrainval.write(name)
        if i in train:
            ftrain.write(name)
        else:
            fval.write(name)
    else:
        ftest.write(name)

ftrainval.close()
ftrain.close()
fval.close()
ftest .close()


"""
事件监听
""" 
from typing import Callable
from fastapi import FastAPI
from ..redis.redis import redis_init  


# app 启动
def create_start_app_handler(app: FastAPI) -> Callable:
    async def start_app() -> None: 
        app.state.log.info('app_start....')
        # 补充APP启动时需要的动作，例如：链接数据库
        # Redis   
        app.state.Redis = await redis_init(app)
    return start_app


# app 停止
def create_stop_app_handler(app: FastAPI) -> Callable:
    async def stop_app() -> None:
        app.state.log.info('app_stop ...')
        # 补充APP关闭时需要的动作：例如关闭数据库
        await app.state.Redis.closed()

    return stop_app



# -*- coding: utf-8 -*-
import sys
from collections.abc import Iterable
from multiprocessing import Pool
from shutil import get_terminal_size

from .timer import Timer


class ProgressBar:
    """A progress bar which can print the progress."""

    def __init__(self, task_num=0, bar_width=50, start=True, file=sys.stdout):
        self.task_num = task_num
        self.bar_width = bar_width
        self.completed = 0
        self.file = file
        if start:
            self.start()

    @property
    def terminal_width(self):
        width, _ = get_terminal_size()
        return width

    def start(self):
        if self.task_num > 0:
            self.file.write(f'[{" " * self.bar_width}] 0/{self.task_num}, '
                            'elapsed: 0s, ETA:')
        else:
            self.file.write('completed: 0, elapsed: 0s')
        self.file.flush()
        self.timer = Timer()

    def update(self, num_tasks=1):
        assert num_tasks > 0
        self.completed += num_tasks
        elapsed = self.timer.since_start()
        if elapsed > 0:
            fps = self.completed / elapsed
        else:
            fps = float('inf')
        if self.task_num > 0:
            percentage = self.completed / float(self.task_num)
            eta = int(elapsed * (1 - percentage) / percentage + 0.5)
            msg = f'\r[{{}}] {self.completed}/{self.task_num}, ' \
                  f'{fps:.1f} task/s, elapsed: {int(elapsed + 0.5)}s, ' \
                  f'ETA: {eta:5}s'

            bar_width = min(self.bar_width,
                            int(self.terminal_width - len(msg)) + 2,
                            int(self.terminal_width * 0.6))
            bar_width = max(2, bar_width)
            mark_width = int(bar_width * percentage)
            bar_chars = '>' * mark_width + ' ' * (bar_width - mark_width)
            self.file.write(msg.format(bar_chars))
        else:
            self.file.write(
                f'completed: {self.completed}, elapsed: {int(elapsed + 0.5)}s,'
                f' {fps:.1f} tasks/s')
        self.file.flush()


def track_progress(func, tasks, bar_width=50, file=sys.stdout, **kwargs):
    """Track the progress of tasks execution with a progress bar.
    Tasks are done with a simple for-loop.
    Args:
        func (callable): The function to be applied to each task.
        tasks (list or tuple[Iterable, int]): A list of tasks or
            (tasks, total num).
        bar_width (int): Width of progress bar.
    Returns:
        list: The task results.
    """
    if isinstance(tasks, tuple):
        assert len(tasks) == 2
        assert isinstance(tasks[0], Iterable)
        assert isinstance(tasks[1], int)
        task_num = tasks[1]
        tasks = tasks[0]
    elif isinstance(tasks, Iterable):
        task_num = len(tasks)
    else:
        raise TypeError(
            '"tasks" must be an iterable object or a (iterator, int) tuple')
    prog_bar = ProgressBar(task_num, bar_width, file=file)
    results = []
    for task in tasks:
        results.append(func(task, **kwargs))
        prog_bar.update()
    prog_bar.file.write('\n')
    return results


def init_pool(process_num, initializer=None, initargs=None):
    if initializer is None:
        return Pool(process_num)
    elif initargs is None:
        return Pool(process_num, initializer)
    else:
        if not isinstance(initargs, tuple):
            raise TypeError('"initargs" must be a tuple')
        return Pool(process_num, initializer, initargs)


def track_parallel_progress(func,
                            tasks,
                            nproc,
                            initializer=None,
                            initargs=None,
                            bar_width=50,
                            chunksize=1,
                            skip_first=False,
                            keep_order=True,
                            file=sys.stdout):
    """Track the progress of parallel task execution with a progress bar.
    The built-in :mod:`multiprocessing` module is used for process pools and
    tasks are done with :func:`Pool.map` or :func:`Pool.imap_unordered`.
    Args:
        func (callable): The function to be applied to each task.
        tasks (list or tuple[Iterable, int]): A list of tasks or
            (tasks, total num).
        nproc (int): Process (worker) number.
        initializer (None or callable): Refer to :class:`multiprocessing.Pool`
            for details.
        initargs (None or tuple): Refer to :class:`multiprocessing.Pool` for
            details.
        chunksize (int): Refer to :class:`multiprocessing.Pool` for details.
        bar_width (int): Width of progress bar.
        skip_first (bool): Whether to skip the first sample for each worker
            when estimating fps, since the initialization step may takes
            longer.
        keep_order (bool): If True, :func:`Pool.imap` is used, otherwise
            :func:`Pool.imap_unordered` is used.
    Returns:
        list: The task results.
    """
    if isinstance(tasks, tuple):
        assert len(tasks) == 2
        assert isinstance(tasks[0], Iterable)
        assert isinstance(tasks[1], int)
        task_num = tasks[1]
        tasks = tasks[0]
    elif isinstance(tasks, Iterable):
        task_num = len(tasks)
    else:
        raise TypeError(
            '"tasks" must be an iterable object or a (iterator, int) tuple')
    pool = init_pool(nproc, initializer, initargs)
    start = not skip_first
    task_num -= nproc * chunksize * int(skip_first)
    prog_bar = ProgressBar(task_num, bar_width, start, file=file)
    results = []
    if keep_order:
        gen = pool.imap(func, tasks, chunksize)
    else:
        gen = pool.imap_unordered(func, tasks, chunksize)
    for result in gen:
        results.append(result)
        if skip_first:
            if len(results) < nproc * chunksize:
                continue
            elif len(results) == nproc * chunksize:
                prog_bar.start()
                continue
        prog_bar.update()
    prog_bar.file.write('\n')
    pool.close()
    pool.join()
    return results


def track_iter_progress(tasks, bar_width=50, file=sys.stdout):
    """Track the progress of tasks iteration or enumeration with a progress
    bar.
    Tasks are yielded with a simple for-loop.
    Args:
        tasks (list or tuple[Iterable, int]): A list of tasks or
            (tasks, total num).
        bar_width (int): Width of progress bar.
    Yields:
        list: The task results.
    """
    if isinstance(tasks, tuple):
        assert len(tasks) == 2
        assert isinstance(tasks[0], Iterable)
        assert isinstance(tasks[1], int)
        task_num = tasks[1]
        tasks = tasks[0]
    elif isinstance(tasks, Iterable):
        task_num = len(tasks)
    else:
        raise TypeError(
            '"tasks" must be an iterable object or a (iterator, int) tuple')
    prog_bar = ProgressBar(task_num, bar_width, file=file)
    for task in tasks:
        yield task
        prog_bar.update()
    prog_bar.file.write('\n')


# -*- coding: utf-8 -*-
import os
from ...settings import config

def get_prefix_uri(_file_):
    basename = os.path.basename(_file_)
    pname = os.path.splitext(basename)[0]
    ves = os.path.abspath(os.path.dirname(_file_)).split('/')[-1].split('\\')[-1]
    api = '/'.join([config.API_PREFIX,ves,pname])
    return api,ves


from setuptools import setup, find_packages 
import codecs
import os
import re

def read(*parts):
    here = os.path.abspath(os.path.dirname(__file__))
    return codecs.open(os.path.join(here, *parts), "r",encoding='utf-8').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(
        r"^VERSION = ['\"]([^'\"]*)['\"]", version_file, re.M
    )
    if version_match:
        return version_match.group(1)

    raise RuntimeError("Unable to find version string.")

setup(
    name='opspipe',
    version=find_version("opspipe","app","settings","config.py"),
    description="This is the ml pipeline",
    author='zhys513',#作者
    author_email="254851907@qq.com",
    url="https://gitee.com/zhys513/opspipe",
    packages=find_packages(exclude=['ops','pipe']), # 排除不生效？
    # 任何包如果包含 *.txt or *.rst 文件都加进去，可以处理多层package目录结构 
    package_data={'': ['*.js','*.css','*.map'],},
    install_requires=['requests'], # 未添加依赖
    python_requires='>=3.6', 
)




# -*- coding: utf-8 -*-
'''
Created on 2020-11-9

@author: zhys513(254851907@qq.com)
'''

from opspipe.reg.common import RegularExtrator 
from unittest import TestCase 
import unittest
 
text = '急寻特朗普，男孩，于2018年11月27号11时在陕西省安康市汉滨区走失。丢失发型短发，...如有线索，请迅速与警方联系：系18100065143，132-6156-2938，baizhantang@sina.com.cn 和yangyangfuture at gmail dot com'

class TestMain(TestCase):
    
    def test_set_txt(self): 
        ex = RegularExtrator(text)
        print("chinese cellphone:")
        print(ex.extract_chinese_cellphone())
        print("email:")
        print(ex.extract_email())
        print("id:")
        print(ex.extract_indentity_ids())
        print("date:")
        print(ex.extract_date())
        print("datetime:")
        print(ex.extract_datetime())

if __name__ == "__main__": 
    unittest.main()
     


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import base64


def base64_to_file(decoded_file_name, base64String):
    # 写入文件
    with open(decoded_file_name, 'wb') as f:
        f.write(base64.b64decode(base64String))
        # print ("解码完毕")
 
def base64_to_str(base64String):
    res = base64.b64decode(base64String.encode('utf-8')).decode("utf-8")
    return res



"""
系统配置
"""
import os
from ..models.db import outModels
# -----------------------系统调试------------------------------------
DEBUG = os.getenv('DEBUG', False)
INTERCEPT = True

# -----------------------项目信息------------------------------------
API_PREFIX = "/aip"
VERSION = "0.0.21"
PROJECT_NAME = "API_UNIFORM_INTERFACE"
DESCRIPTION = 'A simple uniform interface'
MDDOC_URL = '/mddoc'
OPENAPI_JSON = ''

# -----------------------数据库配置-----------------------------------
MYSQL_USER = os.getenv('MYSQL_USER', 'root')
MYSQL_PASSWORD = os.getenv('MYSQL_PASSWORD', '123')
MYSQL_HOST = os.getenv('MYSQL_HOST', 'localhost')
MYSQL_PORT = os.getenv('MYSQL_PORT', 3306)
MYSQL_DB = os.getenv('MYSQL_DB', 'saas')

# -----------------------数据模型-------------------------------------
MODELS = outModels

# -----------------------数据库迁移-----------------------------------
TORTOISE_ORM = {
    "connections": {"default": f"mysql://{MYSQL_USER}:{MYSQL_PASSWORD}@{MYSQL_HOST}:{MYSQL_PORT}/{MYSQL_DB}"},
    "apps": {
        "models": {
            "models": MODELS,
            "default_connection": "default",
        },
    },
}


# -----------------------redis--------------------------------------
REDIS_HOST = os.getenv('REDIS_HOST', '127.0.0.1')
REDIS_PORT = os.getenv('REDIS_PORT', 6379)
REDIS_DB = 0
REDIS_INIT = os.getenv('REDIS_INIT', '') # True、初始化 False、不初始化

# -----------------------跨域支持-------------------------------------
CORS_ORIGINS = ['http://localhost:5000']
CORS_ALLOW_CREDENTIALS = True
CORS_ALLOW_METHODS = ['*']
CORS_ALLOW_HEADERS = ['*']

# -----------------------模板、静态文件---------------------------------
ROOT_PATH = os.getcwd()
SWAGGER = 'swagger'
STATIC_DIR = os.path.join(ROOT_PATH, 'public')
TEMPLATE_DIR = os.path.join(STATIC_DIR, 'template')
MDDOC_DIR = os.path.join(STATIC_DIR, 'mddoc.md')
# -----------------------日志-----------------------------------------
LOG_DIR = os.path.join(ROOT_PATH, 'logs')
 